# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import base64
from unittest import mock

import testtools

from barbican.common import utils
from barbican.model import models
from barbican.plugin.crypto import base
from barbican.plugin.interface import secret_store
from barbican.plugin import store_crypto
from barbican.tests import keys
from barbican.tests import utils as test_utils


def get_private_dto():
    spec = secret_store.KeySpec(secret_store.KeyAlgorithm.RSA, 2048)
    return secret_store.SecretDTO(secret_store.SecretType.PRIVATE,
                                  base64.b64encode(
                                      keys.get_private_key_pem()),
                                  spec,
                                  'application/pkcs8')


def get_public_dto():
    spec = secret_store.KeySpec(secret_store.KeyAlgorithm.RSA, 2048)
    return secret_store.SecretDTO(secret_store.SecretType.PUBLIC,
                                  base64.b64encode(
                                      keys.get_public_key_pem()),
                                  spec,
                                  'application/octet-stream')


def get_certificate_dto():
    spec = secret_store.KeySpec(secret_store.KeyAlgorithm.RSA, 2048)
    return secret_store.SecretDTO(secret_store.SecretType.CERTIFICATE,
                                  base64.b64encode(
                                      keys.get_certificate_pem()),
                                  spec,
                                  'application/octet-stream')


class TestSecretStoreBase(testtools.TestCase,
                          test_utils.MockModelRepositoryMixin):
    """Define common configurations for testing store_crypto.py."""
    def setUp(self):
        super(TestSecretStoreBase, self).setUp()

        self.patchers = []  # List of patchers utilized in this test class.

        self.project_id = '12345'
        self.content_type = 'application/octet-stream'
        self.content_encoding = 'base64'
        self.secret = base64.b64encode(b'secret')
        self.decrypted_secret = b'decrypted_secret'
        self.cypher_text = b'cypher_text'
        self.kek_meta_extended = 'kek-meta-extended'
        self.spec_aes = secret_store.KeySpec('AES', 64, 'CBC')
        self.spec_rsa = secret_store.KeySpec(
            'RSA', 1024, passphrase='changeit')

        self.project_model = mock.MagicMock()
        self.project_model.id = 'project-model-id'
        self.project_model.external_id = self.project_id
        self.secret_dto = secret_store.SecretDTO(
            secret_store.SecretType.OPAQUE,
            self.secret,
            secret_store.KeySpec(),
            self.content_type
        )
        self.response_dto = base.ResponseDTO(
            self.cypher_text, kek_meta_extended=self.kek_meta_extended)
        self.private_key_dto = base.ResponseDTO(self.cypher_text)
        self.public_key_dto = base.ResponseDTO(self.cypher_text)
        self.passphrase_dto = base.ResponseDTO(self.cypher_text)

        self.kek_meta_project_model = models.KEKDatum()
        self.kek_meta_project_model.plugin_name = 'plugin-name'
        self.kek_meta_project_model.kek_label = 'kek-meta-label'
        self.kek_meta_project_model.algorithm = 'kek-meta-algo'
        self.kek_meta_project_model.bit_length = 1024
        self.kek_meta_project_model.mode = 'kek=meta-mode'
        self.kek_meta_project_model.plugin_meta = 'kek-meta-plugin-meta'

        self.encrypted_datum_model = models.EncryptedDatum()
        self.encrypted_datum_model.kek_meta_project = (
            self.kek_meta_project_model)
        self.encrypted_datum_model.cypher_text = base64.b64encode(
            b'cypher_text')
        self.encrypted_datum_model.content_type = 'content_type'
        self.encrypted_datum_model.kek_meta_extended = 'extended_meta'

        self.secret_model = models.Secret(
            {
                'algorithm': 'myalg',
                'bit_length': 1024,
                'mode': 'mymode'
            }
        )
        self.secret_model.id = 'secret-model-id'
        self.secret_model.encrypted_data = [self.encrypted_datum_model]

        self.context = store_crypto.StoreCryptoContext(
            secret_model=self.secret_model,
            project_model=self.project_model,
            content_type=self.content_type)

    def tearDown(self):
        super(TestSecretStoreBase, self).tearDown()
        for patcher in self.patchers:
            patcher.stop()

    def init_patchers(self):
        self._config_get_secret_repository()
        self._config_get_encrypted_datum_repository()
        self._config_get_kek_datum_repository()

    def _start_patcher(self, patcher):
        mock = patcher.start()
        self.patchers.append(patcher)
        return mock

    def _config_get_secret_repository(self):
        """Mock the get_secret_repository() factory function."""
        self.secret_repo = mock.MagicMock()
        self.secret_repo.create_from.return_value = self.secret_model
        self.setup_secret_repository_mock(self.secret_repo)

    def _config_get_encrypted_datum_repository(self):
        """Mock the get_encrypted_datum_repository() factory function."""
        self.datum_repo = mock.MagicMock()
        self.datum_repo.create_from.return_value = None
        self.setup_encrypted_datum_repository_mock(self.datum_repo)

    def _config_get_kek_datum_repository(self):
        """Mock the get_kek_datum_repository() factory function."""
        kek_model = self.kek_meta_project_model
        self.kek_repo = mock.MagicMock()
        self.kek_repo.find_or_create_kek_datum.return_value = kek_model
        self.setup_kek_datum_repository_mock(self.kek_repo)


@test_utils.parameterized_test_case
class WhenTestingStoreCrypto(TestSecretStoreBase):

    dataset_for_pem = {
        'private': [get_private_dto()],
        'public': [get_public_dto()],
        'certificate': [get_certificate_dto()]
    }

    def setUp(self):
        super(WhenTestingStoreCrypto, self).setUp()

        self.init_patchers()
        self._config_crypto_plugin()
        self._config_private_methods()

        self.plugin_to_test = store_crypto.StoreCryptoAdapterPlugin()

    def test_store_secret_with_context_type(self):
        """Test storing a secret."""

        response_dict = self.plugin_to_test.store_secret(
            self.secret_dto, self.context)

        self.assertIsNone(response_dict)

        # Verify encrypt plugin and method where invoked.
        encrypt_mock = self.encrypting_plugin.encrypt
        self.assertEqual(1, encrypt_mock.call_count)
        args, kwargs = encrypt_mock.call_args
        test_encrypt_dto, test_kek_meta_dto, test_project_id = tuple(args)
        self.assertIsInstance(test_encrypt_dto, base.EncryptDTO)
        self.assertEqual(b'secret', test_encrypt_dto.unencrypted)
        self.assertEqual(self.kek_meta_dto, test_kek_meta_dto)
        self.assertEqual(self.project_id, test_project_id)

    def test_store_secret_without_context_type(self):
        """Test storing a secret."""
        self.context.content_type = None

        self.plugin_to_test.store_secret(
            self.secret_dto, self.context)

        self.assertEqual(self.content_type, self.context.content_type)

    @test_utils.parameterized_dataset(dataset_for_pem)
    def test_store_pem_secret(self, secret_dto):
        """Test storing a secret that is PEM encoded."""

        response_dict = self.plugin_to_test.store_secret(
            secret_dto, self.context)

        self.assertIsNone(response_dict)

        raw_content = base64.b64decode(secret_dto.secret)

        # Verify encrypt plugin and method where invoked.
        encrypt_mock = self.encrypting_plugin.encrypt
        self.assertEqual(1, encrypt_mock.call_count)
        args, kwargs = encrypt_mock.call_args
        test_encrypt_dto, test_kek_meta_dto, test_project_id = tuple(args)
        self.assertIsInstance(test_encrypt_dto, base.EncryptDTO)
        self.assertEqual(raw_content, test_encrypt_dto.unencrypted)
        self.assertEqual(self.kek_meta_dto, test_kek_meta_dto)
        self.assertEqual(self.project_id, test_project_id)

    def test_get_secret(self):
        """Test getting a secret."""

        secret_dto = self.plugin_to_test.get_secret(
            secret_store.SecretType.OPAQUE,
            None,  # Secret metadata is not relevant to store_crypto process.
            self.context)

        # Verify response.
        self.assertIsInstance(secret_dto, secret_store.SecretDTO)
        self.assertEqual(secret_store.SecretType.OPAQUE, secret_dto.type)
        self.assertEqual(
            base64.encodebytes(self.decrypted_secret).rstrip(b'\n'),
            secret_dto.secret)
        self.assertEqual(
            self.encrypted_datum_model.content_type, secret_dto.content_type)
        self.assertIsInstance(secret_dto.key_spec, secret_store.KeySpec)
        self.assertEqual(
            self.secret_model.algorithm, secret_dto.key_spec.alg)
        self.assertEqual(
            self.secret_model.bit_length, secret_dto.key_spec.bit_length)
        self.assertEqual(
            self.secret_model.mode, secret_dto.key_spec.mode)

        # Verify decrypt plugin and method where invoked.
        decrypt_mock = self.retrieving_plugin.decrypt
        self.assertEqual(1, decrypt_mock.call_count)
        args, kwargs = decrypt_mock.call_args
        (
            test_decrypt,
            test_kek_meta,
            test_kek_meta_extended,
            test_project_id
        ) = tuple(args)

        self.assertIsInstance(test_decrypt, base.DecryptDTO)
        self.assertEqual(
            base64.b64decode(self.encrypted_datum_model.cypher_text),
            test_decrypt.encrypted)

        self.assertIsInstance(test_kek_meta, base.KEKMetaDTO)
        self.assertEqual(
            self.kek_meta_project_model.plugin_name, test_kek_meta.plugin_name)

        self.assertEqual(
            self.encrypted_datum_model.kek_meta_extended,
            test_kek_meta_extended)

        self.assertEqual(self.project_id, test_project_id)

    @test_utils.parameterized_dataset(dataset_for_pem)
    def test_get_secret_encoding(self, input_secret_dto):
        """Test getting a secret that should be returend in PEM format."""
        secret = input_secret_dto.secret
        key_spec = input_secret_dto.key_spec
        secret_type = input_secret_dto.type

        decrypt_mock = self.retrieving_plugin.decrypt
        decrypt_mock.return_value = base64.decodebytes(secret)

        secret_model = self.context.secret_model
        secret_model.algorithm = key_spec.alg
        secret_model.bit_length = key_spec.bit_length
        secret_model.mode = key_spec.mode

        secret_dto = self.plugin_to_test.get_secret(
            secret_type,
            None,  # Secret metadata is not relevant to store_crypto process.
            self.context)

        # Verify response.
        self.assertIsInstance(secret_dto, secret_store.SecretDTO)
        self.assertEqual(secret, secret_dto.secret)
        self.assertEqual(secret_type, secret_dto.type)
        self.assertIsInstance(secret_dto.key_spec, secret_store.KeySpec)
        self.assertEqual(
            secret_model.algorithm, secret_dto.key_spec.alg)
        self.assertEqual(
            secret_model.bit_length, secret_dto.key_spec.bit_length)
        self.assertEqual(
            secret_model.mode, secret_dto.key_spec.mode)

    def test_generate_symmetric_key(self):
        """test symmetric secret generation."""
        generation_type = base.PluginSupportTypes.SYMMETRIC_KEY_GENERATION
        self._config_determine_generation_type_private_method(
            generation_type)

        response_dict = self.plugin_to_test.generate_symmetric_key(
            self.spec_aes, self.context)

        self.assertIsNone(response_dict)

        # Verify KEK objects finder was invoked.
        method_target = self.find_or_create_kek_objects_patcher.target
        method_mock = method_target._find_or_create_kek_objects
        self.assertEqual(1, method_mock.call_count)

        # Verify generating plugin and method where invoked.
        self._verify_generating_plugin_args(
            self.generating_plugin.generate_symmetric,
            self.spec_aes.alg,
            self.spec_aes.bit_length)

        # Verify secret save was invoked.
        method_target = self.store_secret_and_datum_patcher.target
        method_mock = method_target._store_secret_and_datum
        self.assertEqual(1, method_mock.call_count)

    def test_generate_asymmetric_key_with_passphrase(self):
        """test asymmetric secret generation with passphrase."""
        self._test_generate_asymmetric_key(passphrase='passphrase')

    def test_generate_asymmetric_key_without_passphrase(self):
        """test asymmetric secret generation with passphrase."""
        self._test_generate_asymmetric_key(passphrase=None)

    def test_generate_supports(self):
        """test generate_supports."""
        # False return if KeySpec == None
        self.assertFalse(self.plugin_to_test.generate_supports(None))

        # AES KeySpec should be supported.
        key_spec = secret_store.KeySpec(alg='AES', bit_length=64, mode='CBC')
        self.assertTrue(self.plugin_to_test.generate_supports(key_spec))
        key_spec = secret_store.KeySpec(alg='aes', bit_length=64, mode='CBC')
        self.assertTrue(self.plugin_to_test.generate_supports(key_spec))

        # RSA KeySpec should be supported.
        key_spec = secret_store.KeySpec(alg='RSA', bit_length=2048)
        self.assertTrue(self.plugin_to_test.generate_supports(key_spec))
        # Camellia KeySpec should not be supported.
        self.key_spec = secret_store.KeySpec('Camellia', 64)
        self.assertFalse(self.plugin_to_test.generate_supports(self.key_spec))
        # Bogus KeySpec should not be supported.
        key_spec = secret_store.KeySpec(alg='bogus', bit_length=2048)
        self.assertFalse(self.plugin_to_test.generate_supports(key_spec))

    def test_store_secret_supports(self):
        # All spec types are supported for storage.
        key_spec = secret_store.KeySpec(
            alg='anyalg', bit_length=64, mode='CBC')
        self.assertTrue(self.plugin_to_test.store_secret_supports(key_spec))

    def test_delete_secret(self):
        """Delete is not implemented, so just verify passes."""
        self.plugin_to_test.delete_secret(None)

    def test_should_raise_secret_not_found_get_secret_with_no_model(self):
        self.context.secret_model = None

        self.assertRaises(
            secret_store.SecretNotFoundException,
            self.plugin_to_test.get_secret,
            secret_store.SecretType.OPAQUE,
            None,  # get_secret() doesn't use the secret metadata argument
            self.context
        )

    def test_should_raise_secret_not_found_get_secret_no_encrypted_data(self):
        self.context.secret_model.encrypted_data = []

        self.assertRaises(
            secret_store.SecretNotFoundException,
            self.plugin_to_test.get_secret,
            secret_store.SecretType.OPAQUE,
            None,  # get_secret() doesn't use the secret metadata argument
            self.context
        )

    def test_should_raise_algorithm_not_supported_generate_symmetric_key(self):
        generation_type = base.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION
        self._config_determine_generation_type_private_method(
            generation_type)

        self.assertRaises(
            secret_store.SecretAlgorithmNotSupportedException,
            self.plugin_to_test.generate_symmetric_key,
            self.spec_aes,
            self.context
        )

    def test_should_raise_algo_not_supported_generate_asymmetric_key(self):
        generation_type = base.PluginSupportTypes.SYMMETRIC_KEY_GENERATION
        self._config_determine_generation_type_private_method(
            generation_type)

        self.assertRaises(
            secret_store.SecretAlgorithmNotSupportedException,
            self.plugin_to_test.generate_asymmetric_key,
            self.spec_rsa,
            self.context
        )

    def _test_generate_asymmetric_key(self, passphrase=None):
        """test asymmetric secret generation with passphrase parameter."""
        self.spec_rsa.passphrase = passphrase

        generation_type = base.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION
        self._config_determine_generation_type_private_method(
            generation_type)

        response_dto = self.plugin_to_test.generate_asymmetric_key(
            self.spec_rsa, self.context)

        # Verify response.
        self.assertIsInstance(
            response_dto, secret_store.AsymmetricKeyMetadataDTO)
        self.assertIsNone(response_dto.private_key_meta)
        self.assertIsNone(response_dto.public_key_meta)
        self.assertIsNone(response_dto.passphrase_meta)

        # Verify KEK objects finder was invoked.
        method_target = self.find_or_create_kek_objects_patcher.target
        method_mock = method_target._find_or_create_kek_objects
        self.assertEqual(1, method_mock.call_count)

        # Verify generating plugin and method where invoked.
        self._verify_generating_plugin_args(
            self.generating_plugin.generate_asymmetric,
            self.spec_rsa.alg,
            self.spec_rsa.bit_length)

        # Assert the secret save was called the proper number of times.
        call_count = 2
        if passphrase:
            call_count = 3
        method_target = self.store_secret_and_datum_patcher.target
        method_mock = method_target._store_secret_and_datum
        self.assertEqual(call_count, method_mock.call_count)

    def _verify_generating_plugin_args(self, generate_mock, alg, bit_length):
        """Verify generating plugin and method where invoked."""
        self.assertEqual(1, generate_mock.call_count)
        args, kwargs = generate_mock.call_args
        test_generate_dto, test_kek_meta_dto, test_project_id = tuple(args)
        self.assertIsInstance(test_generate_dto, base.GenerateDTO)
        self.assertEqual(alg, test_generate_dto.algorithm)
        self.assertEqual(bit_length, test_generate_dto.bit_length)
        self.assertEqual(self.kek_meta_dto, test_kek_meta_dto)
        self.assertEqual(self.project_id, test_project_id)

        return generate_mock

    def _config_crypto_plugin(self):
        """Mock the crypto plugin."""

        # Create encrypting and generating plugins (the same plugin does both)
        response_dto = self.response_dto
        self.generating_plugin = mock.MagicMock()
        self.encrypting_plugin = self.generating_plugin
        self.generating_plugin.encrypt.return_value = response_dto
        self.generating_plugin.generate_symmetric.return_value = response_dto
        self.generating_plugin.generate_asymmetric.return_value = (
            self.private_key_dto, self.public_key_dto, self.passphrase_dto
        )

        # Create secret retrieving plugin
        self.retrieving_plugin = mock.MagicMock()
        self.retrieving_plugin.decrypt.return_value = self.decrypted_secret

        gen_plugin_config = {
            'get_plugin_store_generate.return_value': self.generating_plugin,
            'get_plugin_retrieve.return_value': self.retrieving_plugin,
        }
        self.gen_plugin_patcher = mock.patch(
            'barbican.plugin.crypto.manager._PLUGIN_MANAGER',
            **gen_plugin_config
        )
        self._start_patcher(self.gen_plugin_patcher)

    def _config_private_methods(self):
        """Mock store_crypto's private methods."""

        # Mock _find_or_create_kek_objects().
        self.kek_meta_dto = mock.MagicMock()
        find_or_create_kek_objects_config = {
            'return_value': (
                self.kek_meta_project_model, self.kek_meta_dto),
        }
        self.find_or_create_kek_objects_patcher = mock.patch(
            'barbican.plugin.store_crypto._find_or_create_kek_objects',
            **find_or_create_kek_objects_config
        )
        self._start_patcher(self.find_or_create_kek_objects_patcher)

        # Mock _store_secret_and_datum().
        self.store_secret_and_datum_patcher = mock.patch(
            'barbican.plugin.store_crypto._store_secret_and_datum'
        )
        self._start_patcher(self.store_secret_and_datum_patcher)

    def _config_determine_generation_type_private_method(self, type_to_return):
        """Mock _determine_generation_type()."""

        determine_generation_type_config = {
            'return_value': type_to_return,
        }
        self.determine_generation_type_patcher = mock.patch(
            'barbican.plugin.store_crypto._determine_generation_type',
            **determine_generation_type_config
        )
        self._start_patcher(self.determine_generation_type_patcher)


class WhenTestingStoreCryptoDetermineGenerationType(testtools.TestCase):
    """Tests store_crypto.py's _determine_generation_type() function."""

    def test_symmetric_algorithms(self):
        for algorithm in base.PluginSupportTypes.SYMMETRIC_ALGORITHMS:
            self.assertEqual(
                base.PluginSupportTypes.SYMMETRIC_KEY_GENERATION,
                store_crypto._determine_generation_type(algorithm))

        # Case doesn't matter.
        self.assertEqual(
            base.PluginSupportTypes.SYMMETRIC_KEY_GENERATION,
            store_crypto._determine_generation_type('AeS'))

    def test_asymmetric_algorithms(self):
        for algorithm in base.PluginSupportTypes.ASYMMETRIC_ALGORITHMS:
            self.assertEqual(
                base.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION,
                store_crypto._determine_generation_type(algorithm))

        # Case doesn't matter.
        self.assertEqual(
            base.PluginSupportTypes.ASYMMETRIC_KEY_GENERATION,
            store_crypto._determine_generation_type('RsA'))

    def test_should_raise_not_supported_no_algorithm(self):
        self.assertRaises(
            secret_store.SecretAlgorithmNotSupportedException,
            store_crypto._determine_generation_type,
            None
        )

    def test_should_raise_not_supported_bogus_algorithm(self):
        self.assertRaises(
            secret_store.SecretAlgorithmNotSupportedException,
            store_crypto._determine_generation_type,
            'bogus'
        )


class WhenTestingStoreCryptoFindOrCreateKekObjects(TestSecretStoreBase):
    """Tests store_crypto.py's _find_or_create_kek_objects() function."""

    def setUp(self):
        super(WhenTestingStoreCryptoFindOrCreateKekObjects, self).setUp()
        self.init_patchers()
        self._config_private_methods()

    def test_kek_bind_completed(self):
        self.kek_meta_project_model.bind_completed = True
        plugin_inst = self

        kek_model, kek_meta_dto = store_crypto._find_or_create_kek_objects(
            plugin_inst, self.project_model)

        # Verify returns.
        self.assertEqual(self.kek_meta_project_model, kek_model)
        self.assertIsInstance(kek_meta_dto, base.KEKMetaDTO)

        # Verify the KEK repository interactions.
        self._verify_kek_repository_interactions(plugin_inst)

    def test_kek_bind_not_completed(self):
        self.kek_meta_project_model.bind_completed = False
        test_kek_metadata = 'metadata'
        plugin_inst = mock.MagicMock()
        plugin_inst.bind_kek_metadata.return_value = test_kek_metadata

        kek_model, kek_meta_dto = store_crypto._find_or_create_kek_objects(
            plugin_inst, self.project_model)

        # Verify returns.
        self.assertEqual(self.kek_meta_project_model, kek_model)
        self.assertEqual(test_kek_metadata, kek_meta_dto)

        # Verify the KEK repository interactions.
        self._verify_kek_repository_interactions(plugin_inst)

        # Verify bind operations.
        self.assertEqual(
            1, plugin_inst.bind_kek_metadata.call_count)
        self.assertEqual(
            1, self.bind_completed_mock.call_count)
        self.assertEqual(
            1, self.kek_repo.save.call_count)
        args, kwargs = self.kek_repo.save.call_args
        kek_model = args[0]
        self.assertEqual(self.kek_meta_project_model, kek_model)

    def test_kek_raise_no_kek_bind_not_completed(self):
        self.kek_meta_project_model.bind_completed = False
        plugin_inst = mock.MagicMock()
        plugin_inst.bind_kek_metadata.return_value = None

        self.assertRaises(
            base.CryptoKEKBindingException,
            store_crypto._find_or_create_kek_objects,
            plugin_inst,
            self.project_model)

    def _verify_kek_repository_interactions(self, plugin_inst):
        """Verify the KEK repository interactions."""
        self.assertEqual(
            1, self.kek_repo.find_or_create_kek_datum.call_count)
        args, kwargs = self.kek_repo.find_or_create_kek_datum.call_args
        test_project_model = args[0]
        test_full_plugin_name = args[1]
        self.assertEqual(self.project_model, test_project_model)
        plugin_name = utils.generate_fullname_for(plugin_inst)
        self.assertEqual(plugin_name, test_full_plugin_name)

    def _config_private_methods(self):
        """Mock store_crypto's private methods."""

        # Mock _indicate_bind_completed().
        indicate_bind_completed_config = {
            'return_value': None
        }
        self.indicate_bind_completed_patcher = mock.patch(
            'barbican.plugin.store_crypto._indicate_bind_completed',
            **indicate_bind_completed_config)
        self.bind_completed_mock = self._start_patcher(
            self.indicate_bind_completed_patcher)


class WhenTestingStoreCryptoStoreSecretAndDatum(TestSecretStoreBase):
    """Tests store_crypto.py's _store_secret_and_datum() function."""

    def setUp(self):
        super(WhenTestingStoreCryptoStoreSecretAndDatum, self).setUp()

        self.init_patchers()

    def test_without_existing_secret(self):

        self.secret_model.id = None

        store_crypto._store_secret_and_datum(
            self.context,
            self.secret_model,
            self.kek_meta_project_model,
            self.response_dto)

        # Verify the repository interactions.
        self._verify_secret_repository_interactions()
        self._verify_encrypted_datum_repository_interactions()

    def test_with_existing_secret(self):
        store_crypto._store_secret_and_datum(
            self.context,
            self.secret_model,
            self.kek_meta_project_model,
            self.response_dto)

        # Verify the repository interactions.
        self._verify_encrypted_datum_repository_interactions()

        # Verify **not** these repository interactions.
        self.assertEqual(
            0, self.secret_repo.create_from.call_count)

    def _verify_secret_repository_interactions(self):
        """Verify the secret repository interactions."""
        self.assertEqual(
            1, self.secret_repo.create_from.call_count)
        args, kwargs = self.secret_repo.create_from.call_args
        test_secret_model = args[0]
        self.assertEqual(self.secret_model, test_secret_model)

    def _verify_encrypted_datum_repository_interactions(self):
        """Verify the encrypted datum repository interactions."""
        self.assertEqual(
            1, self.datum_repo.create_from.call_count)
        args, kwargs = self.datum_repo.create_from.call_args
        test_datum_model = args[0]
        self.assertIsInstance(test_datum_model, models.EncryptedDatum)
        self.assertEqual(
            self.content_type, test_datum_model.content_type)
        self.assertEqual(
            base64.encodebytes(self.cypher_text).rstrip(b'\n'),
            test_datum_model.cypher_text)
        self.assertEqual(
            self.response_dto.kek_meta_extended,
            test_datum_model.kek_meta_extended)


class WhenTestingStoreCryptoIndicateBindCompleted(TestSecretStoreBase):
    """Tests store_crypto.py's _indicate_bind_completed() function."""

    def test_bind_operation(self):
        kek_meta_dto = base.KEKMetaDTO(self.kek_meta_project_model)
        self.kek_meta_project_model.bind_completed = False

        store_crypto._indicate_bind_completed(
            kek_meta_dto, self.kek_meta_project_model)

        self.assertTrue(self.kek_meta_project_model.bind_completed)
        self.assertEqual(
            self.kek_meta_project_model.algorithm, kek_meta_dto.algorithm)
        self.assertEqual(
            self.kek_meta_project_model.bit_length, kek_meta_dto.bit_length)
        self.assertEqual(
            self.kek_meta_project_model.mode, kek_meta_dto.mode)
        self.assertEqual(
            self.kek_meta_project_model.plugin_meta, kek_meta_dto.plugin_meta)



# Copyright (c) 2017 IBM
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os

from barbican.tests import utils


class WhenTestingContainerConsumersResource(utils.BarbicanAPIBaseTestCase):

    def setUp(self):
        super(WhenTestingContainerConsumersResource, self).setUp()

        self.container_name = "Im_a_container"
        self.container_type = "generic"

        self.consumer_a = {
            "URL": "http://test_a",
            "name": "consumer_a"
        }

        self.consumer_b = {
            "URL": "http://test_b",
            "name": "consumer_b"
        }

        self.consumer_c = {
            "URL": "http://test_c",
            "name": "consumer_c"
        }

    def test_can_create_new_consumer(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_a["name"],
            url=self.consumer_a["URL"]
        )

        self.assertEqual(200, consumer_resp.status_int)
        self.assertEqual([self.consumer_a], consumer)

    def test_can_get_consumers(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_a["name"],
            url=self.consumer_a["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_b["name"],
            url=self.consumer_b["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_c["name"],
            url=self.consumer_c["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_get_resp = self.app.get(
            '/containers/{container_id}/consumers/'.format(
                container_id=container_uuid))

        self.assertEqual(200, consumer_get_resp.status_int)
        self.assertIn(consumers[0]["name"],
                      consumer_get_resp.json["consumers"][0]["name"])
        self.assertIn(consumers[0]["URL"],
                      consumer_get_resp.json["consumers"][0]["URL"])
        self.assertIn(consumers[1]["name"],
                      consumer_get_resp.json["consumers"][1]["name"])
        self.assertIn(consumers[1]["URL"],
                      consumer_get_resp.json["consumers"][1]["URL"])
        self.assertIn(consumers[2]["name"],
                      consumer_get_resp.json["consumers"][2]["name"])
        self.assertIn(consumers[2]["URL"],
                      consumer_get_resp.json["consumers"][2]["URL"])

    def test_can_get_consumers_with_limit_and_offset(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_a["name"],
            url=self.consumer_a["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_b["name"],
            url=self.consumer_b["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_c["name"],
            url=self.consumer_c["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_get_resp = self.app.get(
            '/containers/{container_id}/consumers/?limit=1&offset=1'.format(
                container_id=container_uuid))
        self.assertEqual(200, consumer_get_resp.status_int)

        container_url = resp.json["container_ref"]

        prev_cons = u"{container_url}/consumers?limit=1&offset=0".format(
            container_url=container_url)
        self.assertEqual(prev_cons, consumer_get_resp.json["previous"])

        next_cons = u"{container_url}/consumers?limit=1&offset=2".format(
            container_url=container_url)
        self.assertEqual(next_cons, consumer_get_resp.json["next"])

        self.assertEqual(self.consumer_b["name"],
                         consumer_get_resp.json["consumers"][0]["name"])
        self.assertEqual(self.consumer_b["URL"],
                         consumer_get_resp.json["consumers"][0]["URL"])

        self.assertEqual(3, consumer_get_resp.json["total"])

    def test_can_delete_consumer(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name=self.consumer_a["name"],
            url=self.consumer_a["URL"]
        )
        self.assertEqual(200, consumer_resp.status_int)

        request = {
            'name': self.consumer_a["name"],
            'URL': self.consumer_a["URL"]
        }
        cleaned_request = {key: val for key, val in request.items()
                           if val is not None}

        consumer_del_resp = self.app.delete_json(
            '/containers/{container_id}/consumers/'.format(
                container_id=container_uuid
            ), cleaned_request, headers={'Content-Type': 'application/json'})

        self.assertEqual(200, consumer_del_resp.status_int)

    def test_can_get_no_consumers(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_get_resp = self.app.get(
            '/containers/{container_id}/consumers/'.format(
                container_id=container_uuid))

        self.assertEqual(200, consumer_get_resp.status_int)
        self.assertEqual([], consumer_get_resp.json["consumers"])

    def test_fail_create_container_not_found(self):
        consumer_resp, consumers = create_container_consumer(
            self.app,
            container_id="bad_container_id",
            name=self.consumer_a["name"],
            url=self.consumer_a["URL"],
            expect_errors=True
        )
        self.assertEqual(404, consumer_resp.status_int)

    def test_fail_get_container_not_found(self):
        consumer_get_resp = self.app.get(
            '/containers/{container_id}/consumers/'.format(
                container_id="bad_container_id"), expect_errors=True)

        self.assertEqual(404, consumer_get_resp.status_int)

    def test_fail_delete_container_not_found(self):
        request = {
            'name': self.consumer_a["name"],
            'URL': self.consumer_a["URL"]
        }
        cleaned_request = {key: val for key, val in request.items()
                           if val is not None}

        consumer_del_resp = self.app.delete_json(
            '/containers/{container_id}/consumers/'.format(
                container_id="bad_container_id"
            ), cleaned_request, headers={'Content-Type': 'application/json'},
            expect_errors=True)

        self.assertEqual(404, consumer_del_resp.status_int)

    def test_fail_delete_consumer_not_found(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        request = {
            'name': self.consumer_a["name"],
            'URL': self.consumer_a["URL"]
        }
        cleaned_request = {key: val for key, val in request.items()
                           if val is not None}

        consumer_del_resp = self.app.delete_json(
            '/containers/{container_id}/consumers/'.format(
                container_id=container_uuid
            ), cleaned_request, headers={'Content-Type': 'application/json'},
            expect_errors=True)

        self.assertEqual(404, consumer_del_resp.status_int)

    def test_fail_create_no_name(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_container_consumer(
            self.app,
            container_id=container_uuid,
            url="http://theurl",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_no_url(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name="thename",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_empty_name(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name="",
            url="http://theurl",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_empty_url(self):
        resp, container_uuid = create_container(
            self.app,
            name=self.container_name,
            container_type=self.container_type
        )
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_container_consumer(
            self.app,
            container_id=container_uuid,
            name="thename",
            url="",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)


class WhenTestingSecretConsumersResource(utils.BarbicanAPIBaseTestCase):

    def setUp(self):
        super(WhenTestingSecretConsumersResource, self).setUp()

        self.consumer_a = {
            "service": "service_a",
            "resource_type": "resource_type_a",
            "resource_id": "resource_id_a",
        }

        self.consumer_b = {
            "service": "service_b",
            "resource_type": "resource_type_b",
            "resource_id": "resource_id_b",
        }

        self.consumer_c = {
            "service": "service_c",
            "resource_type": "resource_type_c",
            "resource_id": "resource_id_c",
        }

    def test_can_create_new_consumer(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_a["service"],
            resource_type=self.consumer_a["resource_type"],
            resource_id=self.consumer_a["resource_id"],
        )

        self.assertEqual(200, consumer_resp.status_int)
        self.assertEqual([self.consumer_a], consumer)

    def test_can_get_consumers(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_a["service"],
            resource_type=self.consumer_a["resource_type"],
            resource_id=self.consumer_a["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_b["service"],
            resource_type=self.consumer_b["resource_type"],
            resource_id=self.consumer_b["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_c["service"],
            resource_type=self.consumer_c["resource_type"],
            resource_id=self.consumer_c["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_get_resp = self.app.get(
            '/secrets/{secret_id}/consumers/'.format(
                secret_id=secret_id))

        self.assertEqual(200, consumer_get_resp.status_int)
        self.assertIn(consumers[0]["service"],
                      consumer_get_resp.json["consumers"][0]["service"])
        self.assertIn(consumers[0]["resource_type"],
                      consumer_get_resp.json["consumers"][0]["resource_type"])
        self.assertIn(consumers[0]["resource_id"],
                      consumer_get_resp.json["consumers"][0]["resource_id"])
        self.assertIn(consumers[1]["service"],
                      consumer_get_resp.json["consumers"][1]["service"])
        self.assertIn(consumers[1]["resource_type"],
                      consumer_get_resp.json["consumers"][1]["resource_type"])
        self.assertIn(consumers[1]["resource_id"],
                      consumer_get_resp.json["consumers"][1]["resource_id"])
        self.assertIn(consumers[2]["service"],
                      consumer_get_resp.json["consumers"][2]["service"])
        self.assertIn(consumers[2]["resource_type"],
                      consumer_get_resp.json["consumers"][2]["resource_type"])
        self.assertIn(consumers[2]["resource_id"],
                      consumer_get_resp.json["consumers"][2]["resource_id"])

    def test_can_get_consumers_with_limit_and_offset(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_a["service"],
            resource_type=self.consumer_a["resource_type"],
            resource_id=self.consumer_a["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_b["service"],
            resource_type=self.consumer_b["resource_type"],
            resource_id=self.consumer_b["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_c["service"],
            resource_type=self.consumer_c["resource_type"],
            resource_id=self.consumer_c["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        consumer_get_resp = self.app.get(
            '/secrets/{secret_id}/consumers/?limit=1&offset=1'.format(
                secret_id=secret_id))
        self.assertEqual(200, consumer_get_resp.status_int)

        secret_url = resp.json["secret_ref"]

        prev_cons = u"{secret_url}/consumers?limit=1&offset=0".format(
            secret_url=secret_url)
        self.assertEqual(prev_cons, consumer_get_resp.json["previous"])

        next_cons = u"{secret_url}/consumers?limit=1&offset=2".format(
            secret_url=secret_url)
        self.assertEqual(next_cons, consumer_get_resp.json["next"])

        self.assertEqual(
            self.consumer_b["service"],
            consumer_get_resp.json["consumers"][0]["service"]
        )
        self.assertEqual(
            self.consumer_b["resource_type"],
            consumer_get_resp.json["consumers"][0]["resource_type"]
        )
        self.assertEqual(
            self.consumer_b["resource_id"],
            consumer_get_resp.json["consumers"][0]["resource_id"]
        )

        self.assertEqual(3, consumer_get_resp.json["total"])

    def test_can_delete_consumer(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service=self.consumer_a["service"],
            resource_type=self.consumer_a["resource_type"],
            resource_id=self.consumer_a["resource_id"],
        )
        self.assertEqual(200, consumer_resp.status_int)

        request = {
            "service": self.consumer_a["service"],
            "resource_type": self.consumer_a["resource_type"],
            "resource_id": self.consumer_a["resource_id"],
        }
        cleaned_request = {key: val for key, val in request.items()
                           if val is not None}

        consumer_del_resp = self.app.delete_json(
            '/secrets/{secret_id}/consumers/'.format(
                secret_id=secret_id
            ), cleaned_request, headers={'Content-Type': 'application/json'})

        self.assertEqual(200, consumer_del_resp.status_int)

    def test_can_get_no_consumers(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_get_resp = self.app.get(
            '/secrets/{secret_id}/consumers/'.format(
                secret_id=secret_id))

        self.assertEqual(200, consumer_get_resp.status_int)
        self.assertEqual([], consumer_get_resp.json["consumers"])

    def test_fail_create_secret_not_found(self):
        consumer_resp, consumers = create_secret_consumer(
            self.app,
            secret_id="bad_secret_id",
            service=self.consumer_a["service"],
            resource_type=self.consumer_a["resource_type"],
            resource_id=self.consumer_a["resource_id"],
            expect_errors=True
        )
        self.assertEqual(404, consumer_resp.status_int)

    def test_fail_get_secret_not_found(self):
        consumer_get_resp = self.app.get(
            '/secrets/{secret_id}/consumers/'.format(
                secret_id="bad_secret_id"), expect_errors=True)

        self.assertEqual(404, consumer_get_resp.status_int)

    def test_fail_delete_secret_not_found(self):
        request = {
            "service": self.consumer_a["service"],
            "resource_type": self.consumer_a["resource_type"],
            "resource_id": self.consumer_a["resource_id"],
        }
        cleaned_request = {key: val for key, val in request.items()
                           if val is not None}

        consumer_del_resp = self.app.delete_json(
            '/secrets/{secret_id}/consumers/'.format(
                secret_id="bad_secret_id"
            ), cleaned_request, headers={'Content-Type': 'application/json'},
            expect_errors=True)

        self.assertEqual(404, consumer_del_resp.status_int)

    def test_fail_delete_consumer_not_found(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        request = {
            "service": self.consumer_a["service"],
            "resource_type": self.consumer_a["resource_type"],
            "resource_id": self.consumer_a["resource_id"],
        }
        cleaned_request = {key: val for key, val in request.items()
                           if val is not None}

        consumer_del_resp = self.app.delete_json(
            '/secrets/{secret_id}/consumers/'.format(
                secret_id=secret_id
            ), cleaned_request, headers={'Content-Type': 'application/json'},
            expect_errors=True)

        self.assertEqual(404, consumer_del_resp.status_int)

    def test_fail_create_no_service(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            resource_type="resource_type",
            resource_id="resource_id",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_no_resource_type(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service="service",
            resource_id="resource_id",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_no_resource_id(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service="service",
            resource_type="resource_type",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_empty_service(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service="",
            resource_type="resource_type",
            resource_id="resource_id",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_empty_resource_type(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service="service",
            resource_type="",
            resource_id="resource_id",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)

    def test_fail_create_empty_resource_id(self):
        resp, secret_id = create_secret(self.app)
        self.assertEqual(201, resp.status_int)

        consumer_resp, consumer = create_secret_consumer(
            self.app,
            secret_id=secret_id,
            service="service",
            resource_type="resource_type",
            resource_id="",
            expect_errors=True
        )
        self.assertEqual(400, consumer_resp.status_int)


# ----------------------- Helper Functions ---------------------------
def create_container(app, name=None, container_type=None, expect_errors=False,
                     headers=None):
    request = {
        'name': name,
        'type': container_type
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.post_json(
        '/containers/',
        cleaned_request,
        expect_errors=expect_errors,
        headers=headers
    )

    created_uuid = None
    if resp.status_int == 201:
        container_ref = resp.json.get('container_ref', '')
        _, created_uuid = os.path.split(container_ref)

    return resp, created_uuid


def create_container_consumer(app, container_id=None, name=None, url=None,
                              expect_errors=False, headers=None):
    request = {
        'name': name,
        'URL': url
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.post_json(
        '/containers/{container_id}/consumers/'.format(
            container_id=container_id),
        cleaned_request,
        expect_errors=expect_errors,
        headers=headers
    )

    consumers = None
    if resp.status_int == 200:
        consumers = resp.json.get('consumers', '')

    return resp, consumers


def create_secret(app, expect_errors=False):
    resp = app.post_json('/secrets/', {}, expect_errors=expect_errors)

    secret_id = None
    if resp.status_int == 201:
        secret_ref = resp.json.get('secret_ref', '')
        _, secret_id = os.path.split(secret_ref)

    return resp, secret_id


def create_secret_consumer(app, secret_id=None, service=None,
                           resource_type=None, resource_id=None,
                           expect_errors=False, headers=None):
    request = {
        "service": service,
        "resource_type": resource_type,
        "resource_id": resource_id,
    }
    request = {k: v for k, v in request.items() if v is not None}

    resp = app.post_json(
        "/secrets/{}/consumers/".format(secret_id),
        request,
        expect_errors=expect_errors,
        headers=headers
    )

    consumers = None
    if resp.status_int == 200:
        consumers = resp.json.get('consumers', '')

    return resp, consumers



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_versionedobjects import base as object_base

from barbican.common import exception
from barbican import i18n as u
from barbican.model import models
from barbican.model import repositories as repos
from barbican.objects import base
from barbican.objects import fields


@object_base.VersionedObjectRegistry.register
class ContainerACLUser(base.BarbicanObject, base.BarbicanPersistentObject,
                       object_base.VersionedObjectDictCompat):
    fields = {
        'acl_id': fields.StringField(),
        'user_id': fields.StringField(),
        'status': fields.StringField(nullable=True, default=base.States.ACTIVE)
    }

    db_model = models.ContainerACLUser
    db_repo = repos.get_container_acl_user_repository()

    def _validate_fields(self, change_fields):

        if change_fields.get('user_id') is None:
            msg = u._("Must supply non-None {0} argument for ContainerACLUser "
                      "entry.")
            raise exception.MissingArgumentError(msg.format("user_id"))

    def delete(self, session):
        entity_id = self.id
        self.db_repo.delete_entity_by_id(
            entity_id=entity_id, external_project_id=None, session=session)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import pecan
import webob

from oslo_serialization import jsonutils as json

try:
    import newrelic.agent
    newrelic_loaded = True
except ImportError:
    newrelic_loaded = False

from barbican.model import repositories


class JSONErrorHook(pecan.hooks.PecanHook):
    def on_error(self, state, exc):
        if isinstance(exc, webob.exc.HTTPError):
            exc.body = json.dump_as_bytes({
                'code': exc.status_int,
                'title': exc.title,
                'description': exc.detail
            })
            state.response.content_type = "application/json"
            return exc.body


class BarbicanTransactionHook(pecan.hooks.TransactionHook):
    """Custom hook for Barbican transactions."""
    def __init__(self):
        super(BarbicanTransactionHook, self).__init__(
            start=repositories.start,
            start_ro=repositories.start_read_only,
            commit=repositories.commit,
            rollback=repositories.rollback,
            clear=repositories.clear
        )


class NewRelicHook(pecan.hooks.PecanHook):
    def on_error(self, state, exc):
        if newrelic_loaded:
            newrelic.agent.record_exception()



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Interface to the Alembic migration process and environment.

Concepts in this file are based on Quantum's Alembic approach.

Available Alembic commands are detailed here:
https://alembic.readthedocs.org/en/latest/api.html#module-alembic.command
"""

import os

from alembic import command as alembic_command
from alembic import config as alembic_config

from barbican.common import config
from barbican.common import utils

LOG = utils.getLogger(__name__)


CONF = config.CONF


def init_config(sql_url=None):
    """Initialize and return the Alembic configuration."""
    sqlalchemy_url = sql_url or CONF.sql_connection
    if not sqlalchemy_url:
        raise RuntimeError("Please specify a SQLAlchemy-friendly URL to "
                           "connect to the proper database, either through "
                           "the CLI or the configuration file.")

    if sqlalchemy_url and 'sqlite' in sqlalchemy_url:
        LOG.warning('!!! Limited support for migration commands using'
                    ' sqlite databases; This operation may not succeed.')

    config = alembic_config.Config(
        os.path.join(os.path.dirname(__file__), 'alembic.ini')
    )
    config.barbican_sqlalchemy_url = sqlalchemy_url
    config.set_main_option('script_location',
                           'barbican.model.migration:alembic_migrations')
    return config


def upgrade(to_version='head', sql_url=None):
    """Upgrade to the specified version."""
    alembic_cfg = init_config(sql_url)
    alembic_command.upgrade(alembic_cfg, to_version)


def history(verbose, sql_url=None):
    alembic_cfg = init_config(sql_url)
    alembic_command.history(alembic_cfg, verbose=verbose)


def current(verbose, sql_url=None):
    alembic_cfg = init_config(sql_url)
    alembic_command.current(alembic_cfg, verbose=verbose)


def stamp(to_version='head', sql_url=None):
    """Stamp the specified version, with no migration performed."""
    alembic_cfg = init_config(sql_url)
    alembic_command.stamp(alembic_cfg, to_version)


def generate(autogenerate=True, message='generate changes', sql_url=None):
    """Generate a version file."""
    alembic_cfg = init_config(sql_url)
    alembic_command.revision(alembic_cfg, message=message,
                             autogenerate=autogenerate)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Tasking related information that is shared/common across modules.
"""
from barbican import i18n as u


RETRY_MSEC_DEFAULT = 60 * 1000


class RetryTasks(object):
    """Defines tasks that can be retried/scheduled.

    RPC tasks invoked from the queue are handled via methods on
    barbican.queue.server.Tasks. These calls in turn delegate to the
    'process()' method of BaseTask sub-classes. These calls in turn delegate
    to business logic and plugins via modules in this barbican.tasks package.
    This class defines a common mechanism for the business logic and plugins
    to indicate what RPC tasks need to be retried in a way that the Tasks
    class can interpret as high level RPC tasks to enqueue later.

    In particular the following generic options are available:

    INVOKE_SAME_TASK - Invoke this same task later

    NO_ACTION_REQUIRED - To retry/scheduling actions are required


    The following task/context-specific actions are available:

    INVOKE_CERT_STATUS_CHECK_TASK - Check certificate status later

    """

    INVOKE_SAME_TASK = "Invoke Same Task Again Later"
    NO_ACTION_REQUIRED = "No Retry/Schedule Actions Are Needed"
    INVOKE_CERT_STATUS_CHECK_TASK = "Check Certificate Status Later"


class FollowOnProcessingStatusDTO(object):
    """Follow On Processing status data transfer object (DTO).

    An object of this type is optionally returned by the
    BaseTask.handle_processing() method defined below, and is used to guide
    follow on processing and to provide status feedback to clients.
    """
    def __init__(
        self,
        status=u._('Unknown'),
        status_message=u._('Unknown'),
        retry_task=RetryTasks.NO_ACTION_REQUIRED,
        retry_msec=RETRY_MSEC_DEFAULT
    ):
        """Creates a new FollowOnProcessingStatusDTO.

        :param status: Status for cert order
        :param status_message: Message to explain status type.
        :param retry_msec: Number of milliseconds to wait for retry
        :param retry_task: Task to retry, one of :class:`RetryTasks`
        """
        self.status = status
        self.status_message = status_message
        self.retry_task = retry_task

        if not retry_msec:
            self.retry_msec = 0
        else:
            self.retry_msec = max(int(retry_msec), 0)

    def is_follow_on_needed(self):
        if self.retry_task:
            return RetryTasks.NO_ACTION_REQUIRED != self.retry_task
        else:
            return False



# Copyright (c) 2013-2014 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import base64
import datetime
from unittest import mock

from cryptography.hazmat.backends import default_backend
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.hazmat.primitives import serialization
from OpenSSL import crypto
from oslo_utils import encodeutils

from barbican.common import exception as excep
from barbican.common import hrefs
from barbican.common import resources as res
from barbican.model import models
from barbican.model import repositories
from barbican.plugin.interface import certificate_manager as cert_man
from barbican.plugin.interface import secret_store
from barbican.tasks import certificate_resources as cert_res
from barbican.tasks import common
from barbican.tests import database_utils
from barbican.tests import utils

container_repo = repositories.get_container_repository()
secret_repo = repositories.get_secret_repository()
ca_repo = repositories.get_ca_repository()
project_ca_repo = repositories.get_project_ca_repository()
preferred_ca_repo = repositories.get_preferred_ca_repository()
project_repo = repositories.get_project_repository()
order_repo = repositories.get_order_repository()


class WhenPerformingPrivateOperations(utils.BaseTestCase,
                                      utils.MockModelRepositoryMixin):
    """Tests private methods within certificate_resources.py."""

    def setUp(self):
        super(WhenPerformingPrivateOperations, self).setUp()
        self.order_plugin_meta_repo = mock.MagicMock()
        self.setup_order_plugin_meta_repository_mock(
            self.order_plugin_meta_repo)
        self.order_barbican_meta_repo = mock.MagicMock()
        self.setup_order_barbican_meta_repository_mock(
            self.order_barbican_meta_repo)

    def test_get_plugin_meta(self):
        class Value(object):
            def __init__(self, value):
                self.value = value

        class OrderModel(object):
            id = mock.ANY
            order_plugin_metadata = {
                "foo": Value(1),
                "bar": Value(2),
            }
        order_model = OrderModel()
        self.order_plugin_meta_repo.get_metadata_for_order.return_value = (
            order_model.order_plugin_metadata
        )
        result = cert_res._get_plugin_meta(order_model)

        self._assert_dict_equal(order_model.order_plugin_metadata, result)

    def test_get_plugin_meta_with_empty_dict(self):
        result = cert_res._get_plugin_meta(None)

        self._assert_dict_equal({}, result)

    def test_save_plugin_meta_w_mock_meta(self):
        # Test dict for plugin meta data.
        test_order_model = 'My order model'
        test_plugin_meta = {"foo": 1}

        cert_res._save_plugin_metadata(
            test_order_model, test_plugin_meta)

        self.order_plugin_meta_repo.save.assert_called_once_with(
            test_plugin_meta, test_order_model)

    def test_save_plugin_w_null_meta(self):
        test_order_model = 'My order model'

        # Test None for plugin meta data.
        cert_res._save_plugin_metadata(
            test_order_model, None)

        self.order_plugin_meta_repo.save.assert_called_once_with(
            {}, test_order_model)

    def test_get_barbican_meta_with_empty_dict(self):
        result = cert_res._get_barbican_meta(None)

        self._assert_dict_equal({}, result)

    def test_save_barbican_w_null_meta(self):
        test_order_model = 'My order model'

        # Test None for plugin meta data.
        cert_res._save_barbican_metadata(
            test_order_model, None)

        self.order_barbican_meta_repo.save.assert_called_once_with(
            {}, test_order_model)

    def _assert_dict_equal(self, expected, test):
        self.assertIsInstance(expected, dict)
        self.assertIsInstance(test, dict)

        if expected != test:
            if len(expected) != len(test):
                self.fail('Expected dict not same size as test dict')

            unmatched_items = set(expected.items()) ^ set(test.items())
            if len(unmatched_items):
                self.fail('One or more items different '
                          'between the expected and test dicts')


class BaseCertificateRequestsTestCase(database_utils.RepositoryTestCase):
    """Base Certificate Case Test function """

    def setUp(self):
        super(BaseCertificateRequestsTestCase, self).setUp()

        self.external_project_id = "56789"
        self.project = res.get_or_create_project(self.external_project_id)
        project_repo.save(self.project)

        self.barbican_meta_dto = mock.MagicMock()
        self.order_meta = {}
        self.plugin_meta = {}
        self.barbican_meta = {}
        self.result = cert_man.ResultDTO(
            cert_man.CertificateStatus.WAITING_FOR_CA
        )
        self.result_follow_on = common.FollowOnProcessingStatusDTO()

        self.cert_plugin = mock.MagicMock()
        self.cert_plugin.issue_certificate_request.return_value = self.result
        self.cert_plugin.check_certificate_status.return_value = self.result

        self.store_plugin = mock.MagicMock()

        parsed_ca = {
            'plugin_name': "cert_plugin",
            'plugin_ca_id': "XXXX",
            'name': "test ca",
            'description': 'Test CA',
            'ca_signing_certificate': 'ZZZZZ',
            'intermediates': 'YYYYY'
        }

        self.ca = models.CertificateAuthority(parsed_ca)
        ca_repo.create_from(self.ca)
        self.ca_id = self.ca.id

        # second ca for testing
        parsed_ca = {
            'plugin_name': "cert_plugin",
            'plugin_ca_id': "XXXX2",
            'name': "test ca2",
            'description': 'Test CA2',
            'ca_signing_certificate': 'ZZZZZ2',
            'intermediates': 'YYYYY2'
        }

        self.ca2 = models.CertificateAuthority(parsed_ca)
        ca_repo.create_from(self.ca2)
        self.ca_id2 = self.ca2.id

        # data for preferred CA and global preferred CA tests
        # add those to the repo in those tests
        self.pref_ca = models.PreferredCertificateAuthority(
            self.project.id,
            self.ca_id)

        self.global_pref_ca = models.PreferredCertificateAuthority(
            self.project.id,
            self.ca_id)

        # data for stored key cases
        self.private_key = models.Secret()
        self.private_key.secret_type = 'PRIVATE'
        self.private_key.project_id = self.project.id
        secret_repo.create_from(self.private_key)

        self.public_key = models.Secret()
        self.public_key.secret_type = 'PUBLIC'
        self.public_key.project_id = self.project.id
        secret_repo.create_from(self.public_key)

        self.passphrase = models.Secret()
        self.passphrase.secret_type = 'PASSPHRASE'
        self.passphrase.project_id = self.project.id
        secret_repo.create_from(self.passphrase)

        self.private_key_value = None
        self.public_key_value = "public_key"
        self.passphrase_value = None

        self.parsed_container_with_passphrase = {
            'name': 'container name',
            'type': 'rsa',
            'secret_refs': [
                {'name': 'private_key',
                 'secret_ref': 'https://localhost/secrets/' +
                               self.private_key.id},
                {'name': 'public_key',
                 'secret_ref': 'https://localhost/secrets/' +
                               self.public_key.id},
                {'name': 'private_key_passphrase',
                 'secret_ref': 'https://localhost/secrets/' +
                               self.passphrase.id}
            ]
        }

        self.parsed_container = {
            'name': 'container name',
            'type': 'rsa',
            'secret_refs': [
                {'name': 'private_key',
                 'secret_ref': 'https://localhost/secrets/' +
                               self.private_key.id},
                {'name': 'public_key',
                 'secret_ref': 'https://localhost/secrets/' +
                               self.public_key.id}
            ]
        }

        self.container_with_passphrase = models.Container(
            self.parsed_container_with_passphrase)
        self.container_with_passphrase.project_id = self.project.id
        container_repo.create_from(self.container_with_passphrase)

        self.container = models.Container(self.parsed_container)
        self.container.project_id = self.project.id
        container_repo.create_from(self.container)

        repositories.commit()

        self.stored_key_meta = {
            cert_man.REQUEST_TYPE:
            cert_man.CertificateRequestType.STORED_KEY_REQUEST,
            "container_ref":
            "https://localhost/containers/" + self.container.id,
            "subject_dn": "cn=host.example.com,ou=dev,ou=us,o=example.com"
        }

        self.order = models.Order()
        self.order.meta = self.order_meta
        self.order.project_id = self.project.id
        self.order.order_barbican_meta = self.barbican_meta
        self.order.type = 'certificate'
        order_repo.create_from(self.order)

        self._config_cert_plugin()
        self._config_store_plugin()
        self._config_cert_event_plugin()
        self._config_save_meta_plugin()
        self._config_get_meta_plugin()
        self._config_save_barbican_meta_plugin()
        self._config_get_barbican_meta_plugin()
        self._config_barbican_meta_dto()

    def tearDown(self):
        super(BaseCertificateRequestsTestCase, self).tearDown()
        self.cert_plugin_patcher.stop()
        self.save_plugin_meta_patcher.stop()
        self.get_plugin_meta_patcher.stop()
        self.cert_event_plugin_patcher.stop()
        self.barbican_meta_dto_patcher.stop()
        self.save_barbican_barbican_meta_patcher.stop()
        self.get_barbican_plugin_meta_patcher.stop()
        self.store_plugin_patcher.stop()

    def stored_key_side_effect(self, *args, **kwargs):
        if args[0] == 'PRIVATE':
            return secret_store.SecretDTO(
                secret_store.SecretType.PRIVATE,
                self.private_key_value,
                None,
                'application/octet-string',
                None)
        elif args[0] == 'PASSPHRASE':
            return secret_store.SecretDTO(
                secret_store.SecretType.PASSPHRASE,
                self.passphrase_value,
                None,
                'application/octet-string',
                None)
        elif args[0] == 'PUBLIC':
            return secret_store.SecretDTO(
                secret_store.SecretType.PUBLIC,
                self.public_key_value,
                None,
                'application/octet-string',
                None)
        else:
            return None

    def _test_should_return_waiting_for_ca(self, method_to_test):
        self.result.status = cert_man.CertificateStatus.WAITING_FOR_CA

        method_to_test(
            self.order, self.project, self.result_follow_on)

        self.assertEqual(
            common.RetryTasks.INVOKE_CERT_STATUS_CHECK_TASK,
            self.result_follow_on.retry_task)
        self.assertEqual(
            cert_res.ORDER_STATUS_REQUEST_PENDING.id,
            self.result_follow_on.status)
        self.assertEqual(
            cert_res.ORDER_STATUS_REQUEST_PENDING.message,
            self.result_follow_on.status_message)

    def _test_should_return_certificate_generated(self, method_to_test):
        self.result.status = cert_man.CertificateStatus.CERTIFICATE_GENERATED

        method_to_test(
            self.order, self.project, self.result_follow_on)

        self.assertEqual(
            common.RetryTasks.NO_ACTION_REQUIRED,
            self.result_follow_on.retry_task)
        self.assertEqual(
            cert_res.ORDER_STATUS_CERT_GENERATED.id,
            self.result_follow_on.status)
        self.assertEqual(
            cert_res.ORDER_STATUS_CERT_GENERATED.message,
            self.result_follow_on.status_message)

    def _test_should_raise_client_data_issue_seen(self, method_to_test):
        self.result.status = cert_man.CertificateStatus.CLIENT_DATA_ISSUE_SEEN

        self.assertRaises(
            cert_man.CertificateStatusClientDataIssue,
            method_to_test,
            self.order,
            self.project,
            self.result_follow_on
        )

    def _test_should_raise_status_not_supported(self, method_to_test):
        self.result.status = "Legend of Link"

        self.assertRaises(
            cert_man.CertificateStatusNotSupported,
            method_to_test,
            self.order,
            self.project,
            self.result_follow_on
        )

    def _config_cert_plugin(self):
        """Mock the certificate plugin manager."""
        cert_plugin_config = {
            'return_value.get_plugin.return_value': self.cert_plugin,
            'return_value.get_plugin_by_name.return_value': self.cert_plugin,
            'return_value.get_plugin_by_ca_id.return_value': self.cert_plugin
        }
        self.cert_plugin_patcher = mock.patch(
            'barbican.plugin.interface.certificate_manager'
            '.CertificatePluginManager',
            **cert_plugin_config
        )
        self.cert_plugin_patcher.start()

    def _config_store_plugin(self):
        """Mock the secret store plugin manager."""
        store_plugin_config = {
            'return_value.get_plugin_retrieve_delete.return_value':
            self.store_plugin
        }
        self.store_plugin_patcher = mock.patch(
            'barbican.plugin.interface.secret_store'
            '.get_manager',
            **store_plugin_config
        )
        self.store_plugin_patcher.start()

    def _config_cert_event_plugin(self):
        """Mock the certificate event plugin manager."""
        self.cert_event_plugin_patcher = mock.patch(
            'barbican.plugin.interface.certificate_manager'
            '._EVENT_PLUGIN_MANAGER'
        )
        self.cert_event_plugin_patcher.start()

    def _config_save_meta_plugin(self):
        """Mock the save plugin meta function."""
        self.save_plugin_meta_patcher = mock.patch(
            'barbican.tasks.certificate_resources._save_plugin_metadata'
        )
        self.mock_save_plugin = self.save_plugin_meta_patcher.start()

    def _config_get_meta_plugin(self):
        """Mock the get plugin meta function."""
        get_plugin_config = {'return_value': self.plugin_meta}
        self.get_plugin_meta_patcher = mock.patch(
            'barbican.tasks.certificate_resources._get_plugin_meta',
            **get_plugin_config
        )
        self.get_plugin_meta_patcher.start()

    def _config_save_barbican_meta_plugin(self):
        """Mock the save barbican plugin meta function."""
        self.save_barbican_barbican_meta_patcher = mock.patch(
            'barbican.tasks.certificate_resources._save_barbican_metadata'
        )
        self.mock_barbican_save_plugin = (
            self.save_barbican_barbican_meta_patcher.start()
        )

    def _config_get_barbican_meta_plugin(self):
        """Mock the get barbican plugin meta function."""
        get_barbican_plugin_config = {'return_value': self.barbican_meta}
        self.get_barbican_plugin_meta_patcher = mock.patch(
            'barbican.tasks.certificate_resources._get_barbican_meta',
            **get_barbican_plugin_config
        )
        self.get_barbican_plugin_meta_patcher.start()

    def _config_barbican_meta_dto(self):
        """Mock the BarbicanMetaDTO."""
        get_plugin_config = {'return_value': self.barbican_meta_dto}
        self.barbican_meta_dto_patcher = mock.patch(
            'barbican.plugin.interface.certificate_manager'
            '.BarbicanMetaDTO',
            **get_plugin_config
        )
        self.barbican_meta_dto_patcher.start()


class WhenIssuingCertificateRequests(BaseCertificateRequestsTestCase):
    """Tests the 'issue_certificate_request()' function."""

    def tearDown(self):
        super(WhenIssuingCertificateRequests, self).tearDown()

    def test_should_return_waiting_for_ca(self):
        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

        self._verify_issue_certificate_plugins_called()

    def test_should_return_waiting_for_ca_as_retry(self):
        # For a retry, the plugin-name to look up would have already been
        # saved into the barbican metadata for the order, so just make sure
        # we can retrieve it.
        self.barbican_meta.update({'plugin_name': 'foo-plugin'})
        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

        self._verify_issue_certificate_plugins_called()

    def test_should_return_certificate_generated(self):
        self._test_should_return_certificate_generated(
            cert_res.issue_certificate_request)

        self._verify_issue_certificate_plugins_called()

    def test_should_raise_client_data_issue_seen(self):
        self._test_should_raise_client_data_issue_seen(
            cert_res.issue_certificate_request)

    def _do_pyopenssl_stored_key_request(self):
        self.order_meta.update(self.stored_key_meta)

        pkey = crypto.PKey()
        pkey.generate_key(crypto.TYPE_RSA, 2048)
        key_pem = crypto.dump_privatekey(
            crypto.FILETYPE_PEM, pkey)
        self.private_key_value = base64.b64encode(key_pem)
        self.public_key_value = "public_key"
        self.passphrase_value = None
        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect

        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

    def test_should_return_for_pyopenssl_stored_key(self):
        self._do_pyopenssl_stored_key_request()
        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta.get('generated_csr'))

        # TODO(alee-3) Add tests to validate the request based on the validator
        # code that dave-mccowan is adding.

    def test_should_return_for_openssl_stored_key_ca_id_passed_in(self):
        self.stored_key_meta['ca_id'] = self.ca_id2
        self._do_pyopenssl_stored_key_request()
        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta['generated_csr'])

    def test_should_return_for_openssl_stored_key_pref_ca_defined(self):
        preferred_ca_repo.create_from(self.pref_ca)
        self._do_pyopenssl_stored_key_request()
        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta['generated_csr'])

    def test_should_return_for_openssl_stored_key_global_ca_defined(self):
        preferred_ca_repo.create_from(self.global_pref_ca)
        self._do_pyopenssl_stored_key_request()
        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta['generated_csr'])

    def test_should_return_for_pyopenssl_stored_key_with_passphrase(self):
        self.order_meta.update(self.stored_key_meta)
        self.order_meta['container_ref'] = (
            "https://localhost/containers/" + self.container_with_passphrase.id
        )

        passphrase = "my secret passphrase"
        pkey = crypto.PKey()
        pkey.generate_key(crypto.TYPE_RSA, 2048)
        key_pem = crypto.dump_privatekey(
            crypto.FILETYPE_PEM,
            pkey,
            passphrase=passphrase.encode('utf-8')
        )
        self.private_key_value = base64.b64encode(key_pem)
        self.public_key_value = "public_key"
        self.passphrase_value = base64.b64encode(passphrase.encode('utf-8'))
        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect
        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta['generated_csr'])

        # TODO(alee-3) Add tests to validate the request based on the validator
        # code that dave-mccowan is adding.

    def test_should_return_for_pycrypto_stored_key_with_passphrase(self):
        self.order_meta.update(self.stored_key_meta)
        self.order_meta['container_ref'] = (
            "https://localhost/containers/" + self.container_with_passphrase.id
        )
        passphrase = "my secret passphrase"

        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
        )

        public_key = private_key.public_key()

        private_key_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.
            BestAvailableEncryption(encodeutils.safe_encode(passphrase))
        )
        self.private_key_value = base64.b64encode(private_key_pem)

        public_key_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )
        self.public_key_value = base64.b64encode(public_key_pem)

        self.passphrase_value = base64.b64encode(passphrase.encode('utf-8'))

        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect
        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta['generated_csr'])

        # TODO(alee-3) Add tests to validate the request based on the validator
        # code that dave-mccowan is adding.

    def test_should_return_for_pycrypto_stored_key_without_passphrase(self):
        self.order_meta.update(self.stored_key_meta)

        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
        )

        public_key = private_key.public_key()

        private_key_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        self.private_key_value = base64.b64encode(private_key_pem)

        public_key_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )
        self.public_key_value = base64.b64encode(public_key_pem)

        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect
        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta['generated_csr'])

        # TODO(alee-3) Add tests to validate the request based on the validator
        # code that dave-mccowan is adding.

    def test_should_raise_for_pycrypto_stored_key_no_container(self):
        self.order_meta.update(self.stored_key_meta)

        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
        )

        public_key = private_key.public_key()

        private_key_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        self.private_key_value = base64.b64encode(private_key_pem)

        public_key_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )
        self.public_key_value = base64.b64encode(public_key_pem)

        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect
        self.result.status = cert_man.CertificateStatus.WAITING_FOR_CA
        container_repo.delete_project_entities(self.project.id)

        self.assertRaises(excep.StoredKeyContainerNotFound,
                          cert_res.issue_certificate_request,
                          self.order,
                          self.project,
                          self.result_follow_on)

    def test_should_raise_for_pycrypto_stored_key_no_private_key(self):
        private_key = rsa.generate_private_key(
            public_exponent=65537,
            key_size=2048,
            backend=default_backend()
        )

        public_key = private_key.public_key()

        private_key_pem = private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption()
        )
        self.private_key_value = base64.b64encode(private_key_pem)

        public_key_pem = public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.PKCS1
        )
        self.public_key_value = base64.b64encode(public_key_pem)

        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect
        self.result.status = cert_man.CertificateStatus.WAITING_FOR_CA

        secret_repo.delete_entity_by_id(
            self.private_key.id, self.external_project_id)

        # We need to commit deletions or we'll get deleted objects with deleted
        # set to True.  This is caused by SQLAlchemy's identity mapping and our
        # use of scoped_session.
        repositories.commit()
        self.order.meta.update(self.stored_key_meta)
        self.assertRaises(excep.StoredKeyPrivateKeyNotFound,
                          cert_res.issue_certificate_request,
                          self.order,
                          self.project,
                          self.result_follow_on)

    def test_should_return_for_pyopenssl_stored_key_with_extensions(self):
        self.order_meta.update(self.stored_key_meta)
        pkey = crypto.PKey()
        pkey.generate_key(crypto.TYPE_RSA, 2048)
        self.private_key_value = base64.b64encode(crypto.dump_privatekey(
            crypto.FILETYPE_PEM, pkey))

        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect
        self.order_meta['extensions'] = 'my ASN.1 extensions structure here'
        # TODO(alee-3) Add real extensions data here

        self.result.status = cert_man.CertificateStatus.WAITING_FOR_CA

        cert_res.issue_certificate_request(self.order,
                                           self.project,
                                           self.result_follow_on)

        self._verify_issue_certificate_plugins_called()
        self.assertIsNotNone(self.order.order_barbican_meta['generated_csr'])

        # TODO(alee-3) Add tests to validate the request based on the validator
        # code that dave-mccowan is adding.
        # TODO(alee-3) Add tests to validate the extensions in the request

    def test_should_raise_invalid_operation_seen(self):
        self.result.status = cert_man.CertificateStatus.INVALID_OPERATION

        self.assertRaises(
            cert_man.CertificateStatusInvalidOperation,
            cert_res.issue_certificate_request,
            self.order,
            self.project,
            self.result_follow_on
        )

    def test_should_return_ca_unavailable_for_request(self):
        retry_msec = 123
        status_msg = 'Test status'
        self.result.status = (
            cert_man.CertificateStatus.CA_UNAVAILABLE_FOR_REQUEST)
        self.result.retry_msec = retry_msec
        self.result.status_message = status_msg
        order_ref = hrefs.convert_order_to_href(self.order.id)

        cert_res.issue_certificate_request(self.order,
                                           self.project,
                                           self.result_follow_on)

        self._verify_issue_certificate_plugins_called()

        epm = self.cert_event_plugin_patcher.target._EVENT_PLUGIN_MANAGER
        epm.notify_ca_is_unavailable.assert_called_once_with(
            self.project.id,
            order_ref,
            status_msg,
            retry_msec
        )
        self._verify_issue_certificate_plugins_called()
        self.assertEqual(
            common.RetryTasks.INVOKE_SAME_TASK,
            self.result_follow_on.retry_task)
        self.assertEqual(
            cert_res.ORDER_STATUS_CA_UNAVAIL_FOR_ISSUE.id,
            self.result_follow_on.status)
        self.assertEqual(
            cert_res.ORDER_STATUS_CA_UNAVAIL_FOR_ISSUE.message,
            self.result_follow_on.status_message)

    def test_should_raise_status_not_supported(self):
        self._test_should_raise_status_not_supported(
            cert_res.issue_certificate_request)

    def _verify_issue_certificate_plugins_called(self):
        self.cert_plugin.issue_certificate_request.assert_called_once_with(
            self.order.id,
            self.order_meta,
            self.plugin_meta,
            self.barbican_meta_dto
        )

        self.mock_save_plugin.assert_called_once_with(
            self.order,
            self.plugin_meta
        )

        self.mock_barbican_save_plugin.assert_called_once_with(
            self.order,
            self.barbican_meta
        )


class WhenCheckingCertificateRequests(BaseCertificateRequestsTestCase):
    """Tests the 'check_certificate_request()' function."""

    def setUp(self):
        super(WhenCheckingCertificateRequests, self).setUp()

    def tearDown(self):
        super(WhenCheckingCertificateRequests, self).tearDown()

    def test_should_return_waiting_for_ca(self):
        self._test_should_return_waiting_for_ca(
            cert_res.check_certificate_request)

        self._verify_check_certificate_plugins_called()

    def test_should_return_certificate_generated(self):
        self._test_should_return_certificate_generated(
            cert_res.check_certificate_request)

        self._verify_check_certificate_plugins_called()

    def test_should_raise_client_data_issue_seen(self):
        self._test_should_raise_client_data_issue_seen(
            cert_res.check_certificate_request)

    def test_should_raise_status_not_supported(self):
        self._test_should_raise_status_not_supported(
            cert_res.check_certificate_request)

    def test_should_return_ca_unavailable_for_request(self):
        retry_msec = 123
        status_msg = 'Test status'
        self.result.status = (
            cert_man.CertificateStatus.CA_UNAVAILABLE_FOR_REQUEST)
        self.result.retry_msec = retry_msec
        self.result.status_message = status_msg
        order_ref = hrefs.convert_order_to_href(self.order.id)

        cert_res.check_certificate_request(self.order,
                                           self.project,
                                           self.result_follow_on)

        self._verify_check_certificate_plugins_called()

        epm = self.cert_event_plugin_patcher.target._EVENT_PLUGIN_MANAGER
        epm.notify_ca_is_unavailable.assert_called_once_with(
            self.project.id,
            order_ref,
            status_msg,
            retry_msec
        )
        self.assertEqual(
            common.RetryTasks.INVOKE_SAME_TASK,
            self.result_follow_on.retry_task)
        self.assertEqual(
            cert_res.ORDER_STATUS_CA_UNAVAIL_FOR_CHECK.id,
            self.result_follow_on.status)
        self.assertEqual(
            cert_res.ORDER_STATUS_CA_UNAVAIL_FOR_CHECK.message,
            self.result_follow_on.status_message)

    def _do_pyopenssl_stored_key_request(self):
        self.order_meta.update(self.stored_key_meta)

        pkey = crypto.PKey()
        pkey.generate_key(crypto.TYPE_RSA, 2048)
        key_pem = crypto.dump_privatekey(
            crypto.FILETYPE_PEM, pkey)
        self.private_key_value = base64.b64encode(key_pem)
        self.public_key_value = "public_key"
        self.passphrase_value = None
        self.store_plugin.get_secret.side_effect = self.stored_key_side_effect

        self._test_should_return_waiting_for_ca(
            cert_res.issue_certificate_request)

        self._test_should_return_certificate_generated(
            cert_res.check_certificate_request)

    def test_should_return_for_pyopenssl_stored_key(self):
        self._do_pyopenssl_stored_key_request()
        self._verify_check_certificate_plugins_called()
        self.assertIsNotNone(
            self.order.order_barbican_meta.get('generated_csr'))

    def _verify_check_certificate_plugins_called(self):
        self.cert_plugin.check_certificate_status.assert_called_once_with(
            self.order.id,
            self.order_meta,
            self.plugin_meta,
            self.barbican_meta_dto
        )

        self.mock_save_plugin.assert_called_with(
            self.order,
            self.plugin_meta
        )


class WhenCreatingSubordinateCAs(database_utils.RepositoryTestCase):
    """Tests the 'create_subordinate_ca()' function."""

    def setUp(self):
        super(WhenCreatingSubordinateCAs, self).setUp()
        self.project = res.get_or_create_project('12345')
        self.project2 = res.get_or_create_project('56789')

        self.subject_name = "cn=subca1 signing certificate, o=example.com"
        self.creator_id = "user12345"
        self.name = "Subordinate CA #1"
        self.description = "This is a test subordinate CA"
        self.plugin_name = "dogtag_plugin"

        # create parent ca
        expiration = (datetime.datetime.utcnow() +
                      datetime.timedelta(minutes=10))
        parsed_ca = {'plugin_name': self.plugin_name,
                     'plugin_ca_id': 'ca_master',
                     'expiration': expiration.isoformat(),
                     'name': 'Dogtag CA',
                     'description': 'Master CA for Dogtag plugin',
                     'ca_signing_certificate': 'XXXXX',
                     'intermediates': 'YYYYY'}

        self.parent_ca = models.CertificateAuthority(parsed_ca)
        ca_repo.create_from(self.parent_ca)
        self.parent_ca_ref = 'https://localhost:6311/cas/' + self.parent_ca.id

        self.new_ca_dict = {
            'plugin_ca_id': 'ca_subordinate',
            'expiration': expiration.isoformat(),
            'name': 'Dogtag Subordinate CA',
            'description': 'Subordinate CA for Dogtag plugin',
            'ca_signing_certificate': 'XXXXX',
            'intermediates': 'YYYYY',
        }

        # mock plugin and calls to plugin
        self.cert_plugin = mock.MagicMock()
        self.cert_plugin.supports_create_ca.return_value = True
        self.cert_plugin.create_ca.return_value = self.new_ca_dict
        self._config_cert_plugin()

    def tearDown(self):
        super(WhenCreatingSubordinateCAs, self).tearDown()
        self.cert_plugin_patcher.stop()

    def test_should_create_subordinate_ca(self):
        subca = cert_res.create_subordinate_ca(
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        self.assertIsInstance(subca, models.CertificateAuthority)
        self.assertEqual(self.project.id, subca.project_id)
        self.assertEqual(self.creator_id, subca.creator_id)
        self.assertEqual(self.plugin_name, subca.plugin_name)

    def test_should_raise_invalid_parent_ca(self):
        self.parent_ca_ref = 'https://localhost:6311/cas/' + "BAD-CA-REF"
        self.assertRaises(
            excep.InvalidParentCA,
            cert_res.create_subordinate_ca,
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )

    def test_should_raise_unauthorized_parent_ca(self):
        subca = cert_res.create_subordinate_ca(
            project_model=self.project2,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        subca_ref = hrefs.convert_certificate_authority_to_href(subca.id)
        self.assertRaises(
            excep.UnauthorizedSubCA,
            cert_res.create_subordinate_ca,
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=subca_ref,
            creator_id=self.creator_id)

    def test_should_raise_subcas_not_supported(self):
        self.cert_plugin.supports_create_ca.return_value = False
        self.assertRaises(
            excep.SubCAsNotSupported,
            cert_res.create_subordinate_ca,
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )

    def test_should_raise_subcas_not_created(self):
        self.cert_plugin.create_ca.return_value = None
        self.assertRaises(
            excep.SubCANotCreated,
            cert_res.create_subordinate_ca,
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )

    def test_should_delete_subca(self):
        subca = cert_res.create_subordinate_ca(
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        self.assertIsInstance(subca, models.CertificateAuthority)
        cert_res.delete_subordinate_ca(self.project.external_id, subca)
        self.cert_plugin.delete_ca.assert_called_once_with(subca.plugin_ca_id)

    def test_should_delete_subca_and_all_related_db_entities(self):
        subca = cert_res.create_subordinate_ca(
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        project_ca = models.ProjectCertificateAuthority(
            self.project.id,
            subca.id
        )
        project_ca_repo.create_from(project_ca)
        preferred_ca = models.PreferredCertificateAuthority(
            self.project.id,
            subca.id)
        preferred_ca_repo.create_from(preferred_ca)
        cert_res.delete_subordinate_ca(self.project.external_id, subca)
        self.cert_plugin.delete_ca.assert_called_once_with(subca.plugin_ca_id)

    def test_should_raise_when_delete_pref_subca_with_other_project_ca(self):
        subca = cert_res.create_subordinate_ca(
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        project_ca = models.ProjectCertificateAuthority(
            self.project.id,
            subca.id
        )
        project_ca_repo.create_from(project_ca)
        preferred_ca = models.PreferredCertificateAuthority(
            self.project.id,
            subca.id)
        preferred_ca_repo.create_from(preferred_ca)
        subca2 = cert_res.create_subordinate_ca(
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        project_ca2 = models.ProjectCertificateAuthority(
            self.project.id,
            subca2.id
        )
        project_ca_repo.create_from(project_ca2)
        self.assertRaises(
            excep.CannotDeletePreferredCA,
            cert_res.delete_subordinate_ca,
            self.project.external_id,
            subca
        )

    def test_should_raise_cannot_delete_base_ca(self):
        self.assertRaises(
            excep.CannotDeleteBaseCA,
            cert_res.delete_subordinate_ca,
            self.project.external_id,
            self.parent_ca
        )

    def test_should_raise_unauthorized_subca_delete(self):
        subca = cert_res.create_subordinate_ca(
            project_model=self.project,
            name=self.name,
            description=self.description,
            subject_dn=self.subject_name,
            parent_ca_ref=self.parent_ca_ref,
            creator_id=self.creator_id
        )
        self.assertRaises(
            excep.UnauthorizedSubCA,
            cert_res.delete_subordinate_ca,
            self.project2.external_id,
            subca
        )

    def _config_cert_plugin(self):
        """Mock the certificate plugin manager."""
        cert_plugin_config = {
            'return_value.get_plugin.return_value': self.cert_plugin,
            'return_value.get_plugin_by_name.return_value': self.cert_plugin,
            'return_value.get_plugin_by_ca_id.return_value': self.cert_plugin
        }
        self.cert_plugin_patcher = mock.patch(
            'barbican.plugin.interface.certificate_manager'
            '.CertificatePluginManager',
            **cert_plugin_config
        )
        self.cert_plugin_patcher.start()



# Copyright (c) 2014 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
This test module focuses on typical-flow business logic tests with the
transport key resource classes.
"""

from unittest import mock

import pecan
import webtest

from barbican.api import app
from barbican.api import controllers
from barbican.common import exception as excep
import barbican.context
from barbican.model import models
from barbican.tests import utils


def get_barbican_env(external_project_id):
    class NoopPolicyEnforcer(object):
        def authorize(self, *args, **kwargs):
            return

    kwargs = {'roles': None,
              'user_id': None,
              'project_id': external_project_id,
              'is_admin': True,
              'policy_enforcer': NoopPolicyEnforcer()}
    barbican_env = {'barbican.context':
                    barbican.context.RequestContext(**kwargs)}
    return barbican_env


SAMPLE_TRANSPORT_KEY = """
    -----BEGIN CERTIFICATE-----
    MIIDlDCCAnygAwIBAgIBGDANBgkqhkiG9w0BAQsFADBCMR8wHQYDVQQKDBZ0b21j
    YXQgMjggZG9tYWluIHRyeSAzMR8wHQYDVQQDDBZDQSBTaWduaW5nIENlcnRpZmlj
    YXRlMB4XDTE0MDMyNzA0MTU0OFoXDTE2MDMxNjA0MTU0OFowRTEfMB0GA1UECgwW
    dG9tY2F0IDI4IGRvbWFpbiB0cnkgMzEiMCAGA1UEAwwZRFJNIFRyYW5zcG9ydCBD
    ZXJ0aWZpY2F0ZTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBANEjiTHn
    xWKKnzgBzR8kHo5YKXDbYi01ar0pAiJQ8Xx4MXj3Uf6ckfxvJ7Icb2PhigAgINLe
    td8butAXM0164kHeEMJWI2TG/+2f42Kla2KeU0bdgKbw1egyZreDvhGk/3P46LQt
    LtRBCb5eQWS2gTFocgA5phzRQnmSS4BRTh1MnGxaFLZsPOXqZKptAYaeXyLG63vL
    woBwFVGoodHrRrpYpCd+D6JABBdUEgSCaYG9JBDC5ElSjJnBlCNrUZ2kxokxbsQp
    UHm70LV9c+5n0o1VLJSqnUDuOkoovVWytlKbz0dw0KiTUDjkb4F4D6s+IePV1ufJ
    6cXvXCLLSQa42AcCAwEAAaOBkTCBjjAfBgNVHSMEGDAWgBSiQq7mBrAcTqqsPRvn
    l8pk4uZCWTBGBggrBgEFBQcBAQQ6MDgwNgYIKwYBBQUHMAGGKmh0dHA6Ly9hbGVl
    LXdvcmtwYy5yZWRoYXQuY29tOjgyODAvY2Evb2NzcDAOBgNVHQ8BAf8EBAMCBPAw
    EwYDVR0lBAwwCgYIKwYBBQUHAwIwDQYJKoZIhvcNAQELBQADggEBALmAtjactFHA
    d4nBFpwpwh3tGhkfwoSCuKThX54UXsJawQrx5gaxP0JE7YVLDRe4jn+RHjkXxdxX
    Xt4IugdTsPNq0nvWVAzwZwoGlJZjqghHpD3AB4E5DEoOnVnmJRLFLF0Xg/R5Sw3F
    j9wdVE/hGShrF+fOqNZhTG2Mf4f9TUR1Y8PtoBmtkwnFUoeiaI+Nq6Dd1Qw8ysar
    i/sOzOOjou4vcbYnrKnn2hlSgF6toza0BCGVA8fMyGBh16JtTR1REL7Bf0m3ZQDy
    4hjmPjvUTN3YO2RlLVZXArhhmqcQzCl94P37pAEN/JhAIYvQ2PPM/ofK9XHc9u9j
    rQJGkMpu7ck=
    -----END CERTIFICATE-----"""


def create_transport_key(id_ref="id",
                         plugin_name="default_plugin",
                         transport_key=None):
    """Generate a transport cert entity instance."""
    tkey = models.TransportKey(plugin_name, transport_key)
    tkey.id = id_ref
    return tkey


class FunctionalTest(utils.BaseTestCase):

    def setUp(self):
        super(FunctionalTest, self).setUp()
        root = self.root
        config = {'app': {'root': root}}
        pecan.set_config(config, overwrite=True)
        self.app = webtest.TestApp(pecan.make_app(root))

    def tearDown(self):
        super(FunctionalTest, self).tearDown()
        pecan.set_config({}, overwrite=True)

    @property
    def root(self):
        return controllers.versions.VersionController()


class WhenGettingTransKeysListUsingTransportKeysResource(FunctionalTest):
    def setUp(self):
        super(
            WhenGettingTransKeysListUsingTransportKeysResource, self
        ).setUp()
        self.app = webtest.TestApp(app.build_wsgi_app(self.root))
        self.app.extra_environ = get_barbican_env(self.external_project_id)

    @property
    def root(self):
        self._init()

        class RootController(object):
            transport_keys = controllers.transportkeys.TransportKeysController(
                self.repo)

        return RootController()

    def _init(self):
        self.plugin_name = "default_plugin"
        self.external_project_id = 'keystoneid1234'
        self.params = {'offset': 2, 'limit': 2}

        self.transport_key = SAMPLE_TRANSPORT_KEY
        self.num_keys = 10
        self.offset = 2
        self.limit = 2

        tk_params = {'plugin_name': self.plugin_name,
                     'transport_key': self.transport_key}

        self.tkeys = [create_transport_key(
            id_ref='id' + str(tkid), **tk_params)
            for tkid in range(self.num_keys)]
        self.total = len(self.tkeys)
        self.repo = mock.MagicMock()
        self.repo.get_by_create_date.return_value = (self.tkeys,
                                                     self.offset,
                                                     self.limit,
                                                     self.total)
        self.params = {
            'offset': self.offset,
            'limit': self.limit
        }

    def test_should_get_list_transport_keys(self):
        resp = self.app.get('/transport_keys/',
                            self.params)

        self.repo.get_by_create_date.assert_called_once_with(
            plugin_name=None,
            offset_arg='{0}'.format(self.offset),
            limit_arg='{0}'.format(self.limit),
            suppress_exception=True
        )

        self.assertIn('previous', resp.namespace)
        self.assertIn('next', resp.namespace)
        body = resp.body.decode('utf-8')

        url_nav_next = self._create_url(self.external_project_id,
                                        self.offset + self.limit, self.limit)
        self.assertEqual(1, body.count(url_nav_next))

        url_nav_prev = self._create_url(self.external_project_id,
                                        0, self.limit)
        self.assertEqual(1, body.count(url_nav_prev))

        url_hrefs = self._create_url(self.external_project_id)
        self.assertEqual((self.num_keys + 2), body.count(url_hrefs))

    def test_response_should_include_total(self):
        resp = self.app.get('/transport_keys/',
                            self.params)
        self.assertIn('total', resp.namespace)
        self.assertEqual(self.total, resp.namespace['total'])

    def test_should_handle_no_transport_keys(self):

        del self.tkeys[:]

        resp = self.app.get('/transport_keys/',
                            self.params)

        self.repo.get_by_create_date.assert_called_once_with(
            plugin_name=None,
            offset_arg='{0}'.format(self.offset),
            limit_arg='{0}'.format(self.limit),
            suppress_exception=True
        )

        self.assertNotIn('previous', resp.namespace)
        self.assertNotIn('next', resp.namespace)

    def _create_url(self, external_project_id, offset_arg=None,
                    limit_arg=None):
        if limit_arg:
            offset = int(offset_arg)
            limit = int(limit_arg)
            return '/transport_keys?limit={0}&offset={1}'.format(
                limit, offset)
        else:
            return '/transport_keys'


class WhenCreatingTransKeysListUsingTransportKeysResource(FunctionalTest):
    def setUp(self):
        super(
            WhenCreatingTransKeysListUsingTransportKeysResource, self
        ).setUp()
        self.app = webtest.TestApp(app.build_wsgi_app(self.root))
        self.app.extra_environ = get_barbican_env(self.external_project_id)

    @property
    def root(self):
        self._init()

        class RootController(object):
            transport_keys = controllers.transportkeys.TransportKeysController(
                self.repo)

        return RootController()

    def _init(self):
        self.plugin_name = "default_plugin"
        self.external_project_id = 'keystoneid1234'

        self.repo = mock.MagicMock()
        self.transport_key_req = {
            'plugin_name': self.plugin_name,
            'transport_key': SAMPLE_TRANSPORT_KEY
        }

    def test_should_add_new_transport_key(self):
        resp = self.app.post_json(
            '/transport_keys/',
            self.transport_key_req
        )
        self.assertEqual(201, resp.status_int)

        args, kwargs = self.repo.create_from.call_args
        order = args[0]
        self.assertIsInstance(order, models.TransportKey)

    def test_should_raise_add_new_transport_key_no_secret(self):
        resp = self.app.post_json(
            '/transport_keys/',
            {},
            expect_errors=True
        )
        self.assertEqual(400, resp.status_int)

    def test_should_raise_add_new_transport_key_bad_json(self):
        resp = self.app.post(
            '/transport_keys/',
            '',
            expect_errors=True,
            content_type='application/json'
        )
        self.assertEqual(400, resp.status_int)

    def test_should_raise_add_new_transport_key_no_content_type_header(self):
        resp = self.app.post(
            '/transport_keys/',
            self.transport_key_req,
            expect_errors=True,
        )
        self.assertEqual(415, resp.status_int)


class WhenGettingOrDeletingTransKeyUsingTransportKeyResource(FunctionalTest):

    def setUp(self):
        super(
            WhenGettingOrDeletingTransKeyUsingTransportKeyResource, self
        ).setUp()
        self.app = webtest.TestApp(app.build_wsgi_app(self.root))
        self.app.extra_environ = get_barbican_env(self.external_project_id)

    @property
    def root(self):
        self._init()

        class RootController(object):
            transport_keys = controllers.transportkeys.TransportKeysController(
                self.repo)

        return RootController()

    def _init(self):
        self.external_project_id = 'keystoneid1234'
        self.transport_key = SAMPLE_TRANSPORT_KEY
        self.tkey_id = utils.generate_test_valid_uuid()

        self.tkey = create_transport_key(
            id_ref=self.tkey_id,
            plugin_name="default_plugin",
            transport_key=self.transport_key)

        self.repo = mock.MagicMock()
        self.repo.get.return_value = self.tkey

    def test_should_get_transport_key(self):
        self.app.get('/transport_keys/{0}/'.format(self.tkey.id))

        self.repo.get.assert_called_once_with(entity_id=self.tkey.id)

    def test_should_throw_exception_for_get_when_trans_key_not_found(self):
        self.repo.get.return_value = None
        resp = self.app.get(
            '/transport_keys/{0}/'.format(utils.generate_test_valid_uuid()),
            expect_errors=True
        )
        self.assertEqual(404, resp.status_int)

    def test_should_throw_exception_for_get_when_trans_key_invalid(self):

        resp = self.app.get(
            '/transport_keys/{0}/'.format("invalid_key_id"),
            expect_errors=True
        )
        self.assertEqual(404, resp.status_int)

    def test_should_delete_transport_key(self):
        self.app.delete('/transport_keys/{0}/'.format(self.tkey.id))
        self.repo.delete_entity_by_id.assert_called_once_with(
            entity_id=self.tkey.id,
            external_project_id=self.external_project_id)

    def test_should_throw_exception_for_delete_when_trans_key_not_found(self):
        self.repo.delete_entity_by_id.side_effect = excep.NotFound(
            "Test not found exception")
        resp = self.app.delete(
            '/transport_keys/{0}/'.format(self.tkey.id),
            expect_errors=True
        )
        self.assertEqual(404, resp.status_int)



"""
Copyright 2015 Rackspace

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""
from keystoneclient.v2_0 import client as v2_client
from keystoneclient.v3 import client as v3_client
from requests import auth

STORED_AUTHENTICATION = None


class FunctionalTestAuth(auth.AuthBase):

    def __init__(self, endpoint, version, username, password,
                 project_name, project_domain):
        self.endpoint = endpoint
        self.version = version
        self.username = username
        self.password = password
        self.project_name = project_name
        self.project_domain = project_domain

        self._client = None

    @property
    def service_catalog(self):
        if not self._client:
            self.authenticate()
        return self.stored_auth.get(self.username, {}).get('service_catalog')

    @property
    def auth_client(self):
        if not self._client:
            self.authenticate()
        return self._client

    @property
    def stored_auth(self):
        global STORED_AUTHENTICATION
        if not STORED_AUTHENTICATION:
            STORED_AUTHENTICATION = {}
        return STORED_AUTHENTICATION

    def _auth_with_keystone_client(self):
        if self.version.lower() == 'v2':
            self._client = v2_client.Client(
                username=self.username,
                password=self.password,
                tenant_name=self.project_name,
                auth_url=self.endpoint
            )
            return self._client.auth_token, self._client.tenant_id

        elif self.version.lower() == 'v3':
            self._client = v3_client.Client(
                username=self.username,
                password=self.password,
                user_domain_name=self.project_domain,
                project_name=self.project_name,
                project_domain_name=self.project_domain,
                auth_url=self.endpoint
            )
            return self._client.auth_token, self._client.project_id
        else:
            raise Exception('Unknown authentication version')

    def authenticate(self):
        creds = self.stored_auth.get(self.username)

        if not creds:
            token, project_id = self._auth_with_keystone_client()
            self.stored_auth[self.username] = {
                'token': token,
                'project_id': project_id,
                'service_catalog': self._client.service_catalog,
                'user_id': self._client.auth_user_id
            }

        return self.stored_auth[self.username]

    def get_user_id(self):
        """Return the UID used by keystone to uniquely identify the user"""
        return self.authenticate()['user_id']

    def get_project_id(self):
        """Return the UID used by keystone to identify the user's project"""
        return self.authenticate()['project_id']

    def __call__(self, r):
        creds = self.authenticate()

        # modify and return the request
        r.headers['X-Project-Id'] = creds.get('project_id')
        r.headers['X-Auth-Token'] = creds.get('token')
        return r



# Copyright (c) 2013-2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
API handler for Barbican
"""
import pkgutil

from oslo_policy import policy
from oslo_serialization import jsonutils as json
import pecan

from barbican.common import config
from barbican.common import exception
from barbican.common import utils
from barbican import i18n as u


LOG = utils.getLogger(__name__)
CONF = config.CONF


class ApiResource(object):
    """Base class for API resources."""
    pass


def load_body(req, resp=None, validator=None):
    """Helper function for loading an HTTP request body from JSON.

    This body is placed into into a Python dictionary.

    :param req: The HTTP request instance to load the body from.
    :param resp: The HTTP response instance.
    :param validator: The JSON validator to enforce.
    :return: A dict of values from the JSON request.
    """
    try:
        body = req.body_file.read(CONF.max_allowed_request_size_in_bytes)
        req.body_file.seek(0)
    except IOError:
        LOG.exception("Problem reading request JSON stream.")
        pecan.abort(500, u._('Read Error'))

    try:
        # TODO(jwood): Investigate how to get UTF8 format via openstack
        # jsonutils:
        #     parsed_body = json.loads(raw_json, 'utf-8')
        parsed_body = json.loads(body)
        strip_whitespace(parsed_body)
    except ValueError:
        LOG.exception("Problem loading request JSON.")
        pecan.abort(400, u._('Malformed JSON'))

    if validator:
        try:
            parsed_body = validator.validate(parsed_body)
        except exception.BarbicanHTTPException as e:
            LOG.exception(str(e))
            pecan.abort(e.status_code, e.client_message)

    return parsed_body


def generate_safe_exception_message(operation_name, excep):
    """Generates an exception message that is 'safe' for clients to consume.

    A 'safe' message is one that doesn't contain sensitive information that
    could be used for (say) cryptographic attacks on Barbican. That generally
    means that em.CryptoXxxx should be captured here and with a simple
    message created on behalf of them.

    :param operation_name: Name of attempted operation, with a 'Verb noun'
                           format (e.g. 'Create Secret).
    :param excep: The Exception instance that halted the operation.
    :return: (status, message) where 'status' is one of the webob.exc.HTTP_xxx
                               codes, and 'message' is the sanitized message
                               associated with the error.
    """
    message = None
    reason = None
    status = 500

    try:
        raise excep
    except (policy.PolicyNotAuthorized, policy.InvalidScope):
        message = u._(
            '{operation} attempt not allowed - '
            'please review your '
            'user/project privileges').format(operation=operation_name)
        status = 403

    except exception.BarbicanHTTPException as http_exception:
        reason = http_exception.client_message
        status = http_exception.status_code
    except Exception:
        message = u._('{operation} failure seen - please contact site '
                      'administrator.').format(operation=operation_name)

    if reason:
        message = u._('{operation} issue seen - {reason}.').format(
            operation=operation_name, reason=reason)

    return status, message


@pkgutil.simplegeneric
def get_items(obj):
    """This is used to get items from either a list or a dictionary.

    While false generator is need to process scalar object
    """

    while False:
        yield None


@get_items.register(dict)
def _json_object(obj):
    return obj.items()


@get_items.register(list)
def _json_array(obj):
    return enumerate(obj)


def strip_whitespace(json_data):
    """Recursively trim values from the object passed in using get_items()."""

    for key, value in get_items(json_data):
        if hasattr(value, 'strip'):
            json_data[key] = value.strip()
        else:
            strip_whitespace(value)



# Copyright (c) 2015 Rackspace, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import os
import uuid

from barbican.model import models
from barbican.model import repositories
from barbican.tests import utils
from oslo_utils import uuidutils


order_repo = repositories.get_order_repository()
project_repo = repositories.get_project_repository()
ca_repo = repositories.get_ca_repository()
project_ca_repo = repositories.get_project_ca_repository()
container_repo = repositories.get_container_repository()

generic_key_meta = {
    'name': 'secretname',
    'algorithm': 'AES',
    'bit_length': 256,
    'mode': 'cbc',
    'payload_content_type': 'application/octet-stream'
}


class WhenCreatingOrdersUsingOrdersResource(utils.BarbicanAPIBaseTestCase):

    def test_can_create_a_new_order(self):
        resp, order_uuid = create_order(
            self.app,
            order_type='key',
            meta=generic_key_meta
        )
        self.assertEqual(202, resp.status_int)

        # Make sure we get a valid uuid for the order
        uuid.UUID(order_uuid)

        order = order_repo.get(order_uuid, self.project_id)

        self.assertIsInstance(order, models.Order)

    def test_order_creation_should_allow_unknown_algorithm(self):
        meta = {
            'bit_length': 128,
            'algorithm': 'unknown'
        }
        resp, _ = create_order(
            self.app,
            order_type='key',
            meta=meta
        )

        self.assertEqual(202, resp.status_int)

    def test_order_creation_should_fail_without_a_type(self):
        resp, _ = create_order(
            self.app,
            meta=generic_key_meta,
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_order_creation_should_fail_without_metadata(self):
        resp, _ = create_order(
            self.app,
            order_type='key',
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_order_create_should_fail_w_unsupported_payload_content_type(self):
        meta = {
            'bit_length': 128,
            'algorithm': 'aes',
            'payload_content_type': 'something_unsupported'
        }
        resp, _ = create_order(
            self.app,
            order_type='key',
            meta=meta,
            expect_errors=True
        )

        self.assertEqual(400, resp.status_int)

    def test_order_creation_should_fail_with_bogus_content(self):
        resp = self.app.post(
            '/orders/',
            'random_stuff',
            headers={'Content-Type': 'application/json'},
            expect_errors=True
        )
        self.assertEqual(400, resp.status_int)

    def test_order_creation_should_fail_with_empty_dict(self):
        resp = self.app.post_json(
            '/orders/',
            {},
            headers={'Content-Type': 'application/json'},
            expect_errors=True
        )
        self.assertEqual(400, resp.status_int)

    def test_order_creation_should_fail_without_content_type_header(self):
        resp = self.app.post(
            '/orders/',
            'doesn\'t matter. headers are validated first',
            expect_errors=True,
        )
        self.assertEqual(415, resp.status_int)


class WhenGettingOrdersListUsingOrdersResource(utils.BarbicanAPIBaseTestCase):
    def test_can_get_a_list_of_orders(self):
        # Make sure we have atleast one order to created
        resp, order_uuid = create_order(
            self.app,
            order_type='key',
            meta=generic_key_meta
        )
        self.assertEqual(202, resp.status_int)

        # Get the list of orders
        resp = self.app.get(
            '/orders/',
            headers={'Content-Type': 'application/json'}
        )

        self.assertEqual(200, resp.status_int)
        self.assertIn('total', resp.json)
        self.assertGreater(len(resp.json.get('orders')), 0)

    def test_pagination_attributes_not_available_with_empty_order_list(self):
        params = {'name': 'no_orders_with_this_name'}

        resp = self.app.get(
            '/orders/',
            params
        )

        self.assertEqual(200, resp.status_int)
        self.assertEqual(0, len(resp.json.get('orders')))


class WhenGettingOrDeletingOrders(utils.BarbicanAPIBaseTestCase):
    def test_can_get_order(self):
        # Make sure we have a order to retrieve
        create_resp, order_uuid = create_order(
            self.app,
            order_type='key',
            meta=generic_key_meta
        )
        self.assertEqual(202, create_resp.status_int)

        # Retrieve the order
        get_resp = self.app.get('/orders/{0}/'.format(order_uuid))
        self.assertEqual(200, get_resp.status_int)

    def test_can_delete_order(self):
        # Make sure we have a order to retrieve
        create_resp, order_uuid = create_order(
            self.app,
            order_type='key',
            meta=generic_key_meta
        )
        self.assertEqual(202, create_resp.status_int)

        delete_resp = self.app.delete('/orders/{0}'.format(order_uuid))
        self.assertEqual(204, delete_resp.status_int)

    def test_get_call_on_non_existant_order_should_give_404(self):
        bogus_uuid = uuidutils.generate_uuid()
        resp = self.app.get(
            '/orders/{0}'.format(bogus_uuid),
            expect_errors=True
        )
        self.assertEqual(404, resp.status_int)

    def test_returns_404_on_get_with_bad_uuid(self):
        resp = self.app.get(
            '/orders/98c876d9-aaac-44e4-8ea8-441932962b05X',
            expect_errors=True
        )
        self.assertEqual(404, resp.status_int)

    def test_delete_call_on_non_existant_order_should_give_404(self):
        bogus_uuid = uuidutils.generate_uuid()
        resp = self.app.delete(
            '/orders/{0}'.format(bogus_uuid),
            expect_errors=True
        )
        self.assertEqual(404, resp.status_int)


class WhenCreatingOrders(utils.BarbicanAPIBaseTestCase):
    def test_should_add_new_order(self):
        order_meta = {
            'name': 'secretname',
            'expiration': '2114-02-28T17:14:44.180394',
            'algorithm': 'AES',
            'bit_length': 256,
            'mode': 'cbc',
            'payload_content_type': 'application/octet-stream'
        }
        create_resp, order_uuid = create_order(
            self.app,
            order_type='key',
            meta=order_meta
        )
        self.assertEqual(202, create_resp.status_int)

        order = order_repo.get(order_uuid, self.project_id)
        self.assertIsInstance(order, models.Order)
        self.assertEqual('key', order.type)
        self.assertEqual(order_meta, order.meta)

    def test_should_return_400_when_creating_with_empty_json(self):
        resp = self.app.post_json('/orders/', {}, expect_errors=True)
        self.assertEqual(400, resp.status_int,)

    def test_should_return_415_when_creating_with_blank_body(self):
        resp = self.app.post('/orders/', '', expect_errors=True)
        self.assertEqual(415, resp.status_int)


class WhenPerformingUnallowedOperations(utils.BarbicanAPIBaseTestCase):
    def test_should_not_allow_put_orders(self):
        resp = self.app.put_json('/orders/', expect_errors=True)
        self.assertEqual(405, resp.status_int)

    def test_should_not_allow_delete_orders(self):
        resp = self.app.delete('/orders/', expect_errors=True)
        self.assertEqual(405, resp.status_int)

    def test_should_not_allow_post_order_by_id(self):
        # Create generic order so we don't get a 404 on POST
        resp, order_uuid = create_order(
            self.app,
            order_type='key',
            meta=generic_key_meta
        )
        self.assertEqual(202, resp.status_int)

        resp = self.app.post_json(
            '/orders/{0}'.format(order_uuid),
            {},
            expect_errors=True
        )

        self.assertEqual(405, resp.status_int)


# ----------------------- Helper Functions ---------------------------
def create_order(app, order_type=None, meta=None, expect_errors=False):
    # TODO(jvrbanac): Once test resources is split out, refactor this
    # and similar functions into a generalized helper module and reduce
    # duplication.
    request = {
        'type': order_type,
        'meta': meta
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.post_json(
        '/orders/',
        cleaned_request,
        expect_errors=expect_errors
    )

    created_uuid = None
    if resp.status_int == 202:
        order_ref = resp.json.get('order_ref', '')
        _, created_uuid = os.path.split(order_ref)

    return resp, created_uuid


def create_container(app, name=None, container_type=None, secret_refs=None,
                     expect_errors=False, headers=None):
    request = {
        'name': name,
        'type': container_type,
        'secret_refs': secret_refs if secret_refs else []
    }
    cleaned_request = {key: val for key, val in request.items()
                       if val is not None}

    resp = app.post_json(
        '/containers/',
        cleaned_request,
        expect_errors=expect_errors,
        headers=headers
    )

    created_uuid = None
    if resp.status_int == 201:
        container_ref = resp.json.get('container_ref', '')
        _, created_uuid = os.path.split(container_ref)

    return resp, created_uuid



#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import pecan
from urllib import parse

from barbican.api import controllers
from barbican.api.controllers import containers
from barbican.api.controllers import orders
from barbican.api.controllers import quotas
from barbican.api.controllers import secrets
from barbican.api.controllers import secretstores
from barbican.api.controllers import transportkeys
from barbican.common import utils
from barbican import i18n as u
from barbican import version

LOG = utils.getLogger(__name__)

_MIN_MICROVERSION = 0
_MAX_MICROVERSION = 1
_LAST_UPDATED = '2021-02-10T00:00:00Z'

# NOTE(xek): The above defines the minimum and maximum version of the API
# across all of the v1 REST API.
# When introducing a new microversion, the _MAX_MICROVERSION
# needs to be incremented by 1 and the _LAST_UPDATED string updated.
# Additionally, the new microversion has to be documented in
# doc/source/api/microversion_history.rst
#
# The following is the complete (ordered) list of supported versions
# used by the microversion middleware to parse what is allowed and
# supported.

VERSIONS = ['1.{}'.format(v) for v in range(_MIN_MICROVERSION,
                                            _MAX_MICROVERSION + 1)]
MIN_API_VERSION = VERSIONS[0]
MAX_API_VERSION = VERSIONS[-1]

MIME_TYPE_JSON = 'application/json'
MIME_TYPE_JSON_HOME = 'application/json-home'
MEDIA_TYPE_JSON = 'application/vnd.openstack.key-manager-%s+json'


def is_supported(req, min_version=MIN_API_VERSION,
                 max_version=MAX_API_VERSION):
    """Check if API request version satisfies version restrictions.

    :param req: request object
    :param min_version: minimal version of API needed for correct
           request processing
    :param max_version: maximum version of API needed for correct
           request processing

    :returns: True if request satisfies minimal and maximum API version
             requirements. False in other case.
    """
    requested_version = str(req.environ.get('key-manager.microversion',
                                            MIN_API_VERSION))

    return (VERSIONS.index(max_version) >=
            VERSIONS.index(requested_version) >=
            VERSIONS.index(min_version))


def _version_not_found():
    """Throw exception indicating version not found."""
    pecan.abort(404, u._("The version you requested wasn't found"))


def _get_versioned_url(version):
    if version[-1] != '/':
        version += '/'
    # If host_href is not set in barbican conf, then derive it from request url
    host_part = utils.get_base_url_from_request()
    if host_part[-1] != '/':
        host_part += '/'
    return parse.urljoin(host_part, version)


class BaseVersionController(object):
    """Base class for the version-specific controllers"""

    @classmethod
    def get_version_info(cls, microversion_spec=True):
        version = {
            'id': cls.version_id,
            'status': 'CURRENT',
            'min_version': cls.min_version,
            'max_version': cls.version,
            'links': [
                {
                    'rel': 'self',
                    'href': _get_versioned_url(cls.version_string),
                },
                {
                    'rel': 'describedby',
                    'type': 'text/html',
                    'href': 'https://docs.openstack.org/'
                }
            ],
        }
        if not microversion_spec:
            version.pop('min_version')
            version.pop('max_version')
            version['status'] = 'stable'
            version['updated']: cls.last_updated
            version['media-types'] = [
                {
                    'base': MIME_TYPE_JSON,
                    'type': MEDIA_TYPE_JSON % cls.version_string
                }
            ]
        return version


class V1Controller(BaseVersionController):
    """Root controller for the v1 API"""

    version_string = 'v1'

    # NOTE(jaosorior): We might start using decimals in the future, meanwhile
    # this is the same as the version string.
    version_id = 'v1'

    version = MAX_API_VERSION
    min_version = MIN_API_VERSION
    last_updated = _LAST_UPDATED

    def __init__(self):
        LOG.debug('=== Creating V1Controller ===')
        self.secrets = secrets.SecretsController()
        self.orders = orders.OrdersController()
        self.containers = containers.ContainersController()
        self.transport_keys = transportkeys.TransportKeysController()
        self.quotas = quotas.QuotasController()
        setattr(self, 'project-quotas', quotas.ProjectsQuotasController())
        setattr(self, 'secret-stores', secretstores.SecretStoresController())

    @pecan.expose(generic=True)
    def index(self):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @utils.allow_certain_content_types(MIME_TYPE_JSON, MIME_TYPE_JSON_HOME)
    @controllers.handle_exceptions(u._('Version retrieval'))
    def on_get(self):
        pecan.core.override_template('json')
        if is_supported(pecan.request, max_version='1.0'):
            return {'version': self.get_version_info(microversion_spec=False)}
        else:
            return {'version': self.get_version_info()}


AVAILABLE_VERSIONS = {
    V1Controller.version_string: V1Controller,
}

DEFAULT_VERSION = V1Controller.version_string


class VersionsController(object):

    def __init__(self):
        LOG.debug('=== Creating VersionsController ===')

    @pecan.expose(generic=True)
    def index(self, **kwargs):
        pecan.abort(405)  # HTTP 405 Method Not Allowed as default

    @index.when(method='GET', template='json')
    @utils.allow_certain_content_types(MIME_TYPE_JSON, MIME_TYPE_JSON_HOME)
    def on_get(self, **kwargs):
        """The list of versions is dependent on the context."""
        self._redirect_to_default_json_home_if_needed(pecan.request)

        if 'build' in kwargs:
            return {'build': version.__version__}

        if is_supported(pecan.request, max_version='1.0'):
            resp = {
                'versions': {
                    'values': [
                        V1Controller.get_version_info(microversion_spec=False)
                    ]
                }
            }
        else:
            resp = {
                'versions': [
                    version_cls.get_version_info() for version_cls in
                    AVAILABLE_VERSIONS.values()]
            }

        # Since we are returning all the versions available, the proper status
        # code is Multiple Choices (300)
        pecan.response.status = 300
        return resp

    def _redirect_to_default_json_home_if_needed(self, request):
        if self._mime_best_match(request.accept) == MIME_TYPE_JSON_HOME:
            url = _get_versioned_url(DEFAULT_VERSION)
            LOG.debug("Redirecting Request to " + url)
            # NOTE(jaosorior): This issues an "external" redirect because of
            # two reasons:
            # * This module doesn't require authorization, and accessing
            #   specific version info needs that.
            # * The resource is a separate app_factory and won't be found
            #   internally
            pecan.redirect(url, request=request)

    def _mime_best_match(self, accept):
        if not accept:
            return MIME_TYPE_JSON

        SUPPORTED_TYPES = [MIME_TYPE_JSON, MIME_TYPE_JSON_HOME]
        return accept.best_match(SUPPORTED_TYPES)



# Copyright (c) 2014 Hewlett-Packard Development Company, L.P.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""
Server-side (i.e. worker side) Keystone notification related classes and logic.
"""
import oslo_messaging
from oslo_service import service

from barbican.common import utils
from barbican.model import repositories
from barbican import queue
from barbican.tasks import keystone_consumer


LOG = utils.getLogger(__name__)


class NotificationTask(object):
    """Task which exposes the API for consuming priority based notifications.

    The Oslo notification framework delivers notifications based on priority to
    matching callback APIs as defined in its notification listener endpoint
    list.

    Currently from Keystone perspective, `info` API is sufficient as Keystone
    send notifications at `info` priority ONLY. Other priority level APIs
    (warn, error, critical, audit, debug) are not needed here.
    """

    def __init__(self, conf):
        self.conf = conf
        repositories.setup_database_engine_and_factory()

    def info(self, ctxt, publisher_id, event_type, payload, metadata):
        """Receives notification at info level."""
        return self.process_event(ctxt, publisher_id, event_type, payload,
                                  metadata)

    def process_event(self, ctxt, publisher_id, event_type, payload, metadata):
        """Process Keystone Event based on event_type and payload data.

        Parses notification data to identify if the event is related to delete
        project or not. In case of delete project event, it passes project_id
        to KeystoneEventConsumer logic for further processing. Barbican service
        is not interested in other events so in that case it just returns None
        as acknowledgment.

        Messaging server considers message is acknowledged when either return
        value is `oslo_messaging.NotificationResult.HANDLED` or None.

        In case of successful processing of notification, the returned value is
        `oslo_messaging.NotificationResult.HANDLED`

        In case of notification processing error, the value returned
        is oslo_messaging.NotificationResult.REQUEUE when transport
        supports this feature otherwise
        `oslo_messaging.NotificationResult.HANDLED` is returned.

        """

        LOG.debug("Input keystone event publisher_id = %s", publisher_id)
        LOG.debug("Input keystone event payload = %s", payload)
        LOG.debug("Input keystone event type = %s", event_type)
        LOG.debug("Input keystone event metadata = %s", metadata)
        project_id = self._parse_payload_for_project_id(payload)
        resource_type, operation_type = self._parse_event_type(event_type)
        LOG.debug('Keystone Event: resource type={0}, operation type={1}, '
                  'keystone id={2}'.format(resource_type, operation_type,
                                           project_id))

        if (project_id and resource_type == 'project' and
                operation_type == 'deleted'):

            task = keystone_consumer.KeystoneEventConsumer()
            try:
                task.process(project_id=project_id,
                             resource_type=resource_type,
                             operation_type=operation_type)
                return oslo_messaging.NotificationResult.HANDLED
            except Exception:
                # No need to log message here as task process method has
                # already logged it
                # TODO(john-wood-w) This really should be retried on a
                #   schedule and really only if the database is down, not
                #   for any exception otherwise tasks will be re-queued
                #   repeatedly. Revisit as part of the retry task work later.
                if self.conf.keystone_notifications.allow_requeue:
                    return oslo_messaging.NotificationResult.REQUEUE
                else:
                    return oslo_messaging.NotificationResult.HANDLED
        return None  # in case event is not project delete

    def _parse_event_type(self, event_type):
        """Parses event type provided as part of notification.

        Parses to identify what operation is performed and on which Keystone
        resource.

        A few event type sample values are provided below::
            identity.project.deleted
            identity.role.created
            identity.domain.updated
            identity.authenticate
        """
        resource_type = None
        operation_type = None
        if event_type:
            type_list = event_type.split('.')
            # 2 is min. number of dot delimiters expected in event_type value.
            if len(type_list) > 2:
                resource_type = type_list[-2].lower()
                operation_type = type_list[-1].lower()

        return resource_type, operation_type

    def _parse_payload_for_project_id(self, payload_s):
        """Gets project resource identifier from payload

        Sample payload is provided below::
            {'resource_info': '2b99a94ad02741978e613fb52dd1f4cd'}
        """
        if payload_s:
            return payload_s.get('resource_info')


class MessageServer(NotificationTask, service.Service):
    """Server to retrieve messages from queue used by Keystone.

    This is used to send public notifications for openstack service
    consumption.

    This server is an Oslo notification server that exposes set of standard
    APIs for events consumption based on event priority.

    Some of messaging server configuration needs to match with Keystone
    deployment notification configuration e.g. exchange name, topic name
    """
    def __init__(self, conf):
        pool_size = conf.keystone_notifications.thread_pool_size
        NotificationTask.__init__(self, conf)
        service.Service.__init__(self, threads=pool_size)

        self.target = queue.get_notification_target()
        self._msg_server = queue.get_notification_server(targets=[self.target],
                                                         endpoints=[self])

    def start(self):
        self._msg_server.start()
        super(MessageServer, self).start()

    def stop(self):
        super(MessageServer, self).stop()
        self._msg_server.stop()
        queue.cleanup()



#    Copyright 2018 Fujitsu.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from barbican.objects import base
from barbican.objects import container
from barbican.objects import container_acl
from barbican.objects import container_consumer_meta
from barbican.objects import container_secret
from barbican.objects import encrypted_datum
from barbican.objects import kekdatum
from barbican.objects import order
from barbican.objects import order_barbican_metadatum
from barbican.objects import order_plugin_metadatum
from barbican.objects import order_retry_task
from barbican.objects import project
from barbican.objects import project_quotas
from barbican.objects import project_secret_store
from barbican.objects import secret
from barbican.objects import secret_acl
from barbican.objects import secret_consumer_metadatum
from barbican.objects import secret_store_metadatum
from barbican.objects import secret_stores
from barbican.objects import secret_user_metadatum
from barbican.objects import transport_key

States = base.States
BarbicanObject = base.BarbicanObject
Container = container.Container
ContainerACL = container_acl.ContainerACL
ContainerConsumerMetadatum = container_consumer_meta.ContainerConsumerMetadatum
ContainerSecret = container_secret.ContainerSecret
EncryptedDatum = encrypted_datum.EncryptedDatum
Order = order.Order
OrderBarbicanMetadatum = order_barbican_metadatum.OrderBarbicanMetadatum
OrderPluginMetadatum = order_plugin_metadatum.OrderPluginMetadatum
OrderRetryTask = order_retry_task.OrderRetryTask
Project = project.Project
ProjectQuotas = project_quotas.ProjectQuotas
ProjectSecretStore = project_secret_store.ProjectSecretStore
TransportKey = transport_key.TransportKey
KEKDatum = kekdatum.KEKDatum
Secret = secret.Secret
SecretACL = secret_acl.SecretACL
SecretStores = secret_stores.SecretStores
SecretUserMetadatum = secret_user_metadatum.SecretUserMetadatum
SecretStoreMetadatum = secret_store_metadatum.SecretStoreMetadatum
SecretConsumerMetadatum = secret_consumer_metadatum.SecretConsumerMetadatum

__all__ = (
    States,
    BarbicanObject,
    Container,
    ContainerACL,
    ContainerConsumerMetadatum,
    ContainerSecret,
    EncryptedDatum,
    Order,
    OrderBarbicanMetadatum,
    OrderPluginMetadatum,
    OrderRetryTask,
    Project,
    ProjectQuotas,
    ProjectSecretStore,
    KEKDatum,
    Secret,
    SecretACL,
    SecretStores,
    SecretUserMetadatum,
    SecretStoreMetadatum,
    SecretConsumerMetadatum,
    TransportKey,
)



# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import sys

sys.path.insert(0, os.path.abspath('../..'))
# -- General configuration ----------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.graphviz',
    # 'sphinx.ext.intersphinx',
    'openstackdocstheme',
    'oslo_config.sphinxext',
    'oslo_config.sphinxconfiggen',
    'oslo_policy.sphinxext',
    'oslo_policy.sphinxpolicygen',
    'sphinxcontrib.rsvgconverter',
]

config_generator_config_file = [
    ('../../etc/oslo-config-generator/barbican.conf',
     '_static/barbican'),
]

policy_generator_config_file = '../../etc/oslo-config-generator/policy.conf'
sample_policy_basename = '_static/barbican'

# autodoc generation is a bit aggressive and a nuisance when doing heavy
# text edit cycles.
# execute "export SPHINX_DEBUG=1" in your terminal to disable

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = 'Barbican'
copyright = '2014, OpenStack Foundation'
openstackdocs_repo_name = 'openstack/barbican'
openstackdocs_pdf_link = True
openstackdocs_auto_name = False
openstackdocs_bug_project = 'barbican'
openstackdocs_bug_tag = ''

# Add any paths that contain "extra" files, such as .htaccess or
# robots.txt.
html_extra_path = ['_extra']

# If true, '()' will be appended to :func: etc. cross-reference text.
add_function_parentheses = True

# If true, the current module name will be prepended to all description
# unit titles (such as .. function::).
add_module_names = True

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'native'

# -- Options for HTML output --------------------------------------------------

# The theme to use for HTML and HTML Help pages.  Major themes that come with
# Sphinx are currently 'default' and 'sphinxdoc'.
# html_theme_path = ["."]
# html_theme = '_theme'
html_theme = 'openstackdocs'
html_static_path = ['_static']
html_theme_options = {}

# Output file base name for HTML help builder.
htmlhelp_basename = '%sdoc' % project

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title, author, documentclass
# [howto/manual]).
latex_documents = [
    ('index',
     'doc-barbican.tex',
     '%s Documentation' % project,
     'OpenStack Foundation', 'manual'),
]

latex_use_xindy = False

# Example configuration for intersphinx: refer to the Python standard library.
# intersphinx_mapping = {'http://docs.python.org/': None}
